# Generated by Django 4.2.14 on 2024-08-02 16:02

import datetime
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import mini_buildd.config
import mini_buildd.models
import mini_buildd.util


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('mini_buildd', '0033_migrate_distribution_lintian'),
    ]

    operations = [
        migrations.AlterField(
            model_name='aptkey',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default='', help_text='<pre>Actual (ASCII-armored) GnuPG public key (leave ``key_id`` above blank)</pre>'),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_created',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_expires',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_id',
            field=models.CharField(blank=True, default='', help_text='<pre>Key id to retrieve the actual key per key server (leave ``key`` below blank)</pre>', max_length=100, validators=[django.core.validators.MinLengthValidator(8, message='The key id, if given, must be at least 8 bytes long')]),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_long_id',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_name',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='architectureoption',
            name='build_architecture_all',
            field=models.BooleanField(default=False, help_text='<pre>Use to build arch-all packages</pre>'),
        ),
        migrations.AlterField(
            model_name='architectureoption',
            name='optional',
            field=models.BooleanField(default=False, help_text='<pre>Don&#x27;t care if the architecture can&#x27;t be build</pre>'),
        ),
        migrations.AlterField(
            model_name='archive',
            name='url',
            field=models.URLField(default='http://ftp.debian.org/debian/', help_text='<pre>The URL of an APT archive (there must be a &#x27;dists/&#x27; infrastructure below)\n\nUse the &#x27;directory&#x27; notation with exactly one trailing slash (like &#x27;http://example.org/path/&#x27;).</pre>', max_length=512, primary_key=True, serialize=False, validators=[mini_buildd.models.RegexValidator('^.*[^/]+/$', message="Must have exactly one trailing slash (like 'http://example.org/path/')")]),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='architecture',
            field=models.ForeignKey(help_text='<pre>Chroot&#x27;s architecture\n\nUsing the same arch as the host system will always work, other architectures may work if supported. An &#x27;amd64&#x27; host, for example, will also allow for architecture &#x27;i386&#x27;.\n</pre>', on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.architecture'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='debootstrap_command',
            field=models.CharField(default='/usr/sbin/debootstrap', help_text='<pre>bootstrap command\n\nChange only if you need an alternate debootstrap command.\n\nFor example, ``/usr/sbin/mini-buildd-debootstrap-uname-2.6`` may be used as workaround for urold (``&lt;= lenny``) distributions.</pre>', max_length=255),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='debootstrap_extra_options',
            field=models.CharField(blank=True, default='', help_text='<pre>Additional debootstrap options\n\nFor example, ``--include=foo,bar`` may be used to have these packages pre-installed in build chroots.\n</pre>', max_length=255),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='personality',
            field=models.CharField(blank=True, default='', editable=False, help_text='<pre>Schroot &#x27;personality&#x27; value (see &#x27;schroot&#x27;)\n\nFor 32bit chroots running on a 64bit host, this must be &#x27;linux32&#x27;.</pre>', max_length=50),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='personality_override',
            field=models.CharField(blank=True, default='', help_text='<pre>Leave empty unless you want to override the automated way (via\nan internal mapping). Please report manual overrides so it can\ngo to the default mapping.\n</pre>', max_length=50),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='source',
            field=models.ForeignKey(help_text='<pre>Base source to create the chroot from; its codename must be &#x27;debootstrapable&#x27;\n\nExamples: Debian &#x27;squeeze&#x27;, Debian &#x27;wheezy&#x27;, Ubuntu &#x27;hardy&#x27;.</pre>', on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='allow_emails_to',
            field=models.CharField(default=mini_buildd.config.default_allow_emails_to, help_text='<pre>Regex to allow sending emails to automatically computed addresses\n\nCurrently the &#x27;Changed-By&#x27; or &#x27;Maintainer&#x27; addresses from the changelog when this feature is enabled for the resp. repository.\n\nUse &#x27;.*&#x27; to allow all -- it&#x27;s however recommended to put this to s.th. like &#x27;.*@myemail.domain&#x27;, to avoid original package maintainers to be accidentally spammed.\n</pre>', max_length=254),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='custom_archive_origin',
            field=models.CharField(blank=True, default='', help_text='<pre>Custom APT &#x27;Origin&#x27; field for repository indices</pre>', max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='email_address',
            field=mini_buildd.models.DaemonEmailField(default=mini_buildd.config.default_email_address, help_text='<pre>mini-buildd instance&#x27;s email address\n\nWill be used in:\n\n* &#x27;Name-Email&#x27; part of the GnuPG key.\n* Sender for notification email.\n</pre>', max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='event_queue_size',
            field=models.IntegerField(default=500, help_text='<pre>[Daemon must be restarted to activate this setting.]\n\nMax number of items that can be stored in the events queue</pre>'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='expire_builds_after',
            field=models.DurationField(default=datetime.timedelta(days=5), help_text='<pre>How long to keep builds and associated files\n\nFormat hint: &lt;DAYS&gt; &lt;HOURS&gt;:&lt;MINUTES&gt;:&lt;SECONDS&gt;</pre>'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='expire_events_after',
            field=models.DurationField(default=datetime.timedelta(days=336), help_text='<pre>How long to keep events and associated files (like buildlogs)\n\nFormat hint: &lt;DAYS&gt; &lt;HOURS&gt;:&lt;MINUTES&gt;:&lt;SECONDS&gt;</pre>'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='ftpd_bind',
            field=models.CharField(default='tcp:port=8067', help_text='<pre>[Daemon must be restarted to activate this setting.]\n\nFTP Server Endpoint: Twisted-style Network Server Endpoint\n\nSee: https://twistedmatrix.com/documents/current/core/howto/endpoints.html#servers\n\nNotation Examples::\n\n  &#x27;tcp6:port=8066&#x27;                                     : Unencrypted, bind on all interfaces.\n  &#x27;ssl:port=8066:privateKey=KEY_PATH:certKey=CERT_PATH&#x27;: Encrypted, bind on all interfaces.\n  &#x27;tcp6:interface=localhost:port=8066&#x27;                 : Unencrypted, bind on localhost only.</pre>', max_length=200, validators=[mini_buildd.models.ServerEndpointValidator()]),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='ftpd_options',
            field=models.CharField(blank=True, default='', help_text='<pre>[Daemon must be restarted to activate this setting.]\n\nftpd handler options (see ``https://pyftpdlib.readthedocs.io/en/latest/api.html#control-connection``).\n\nFormat is ``name0=value0;name1=value1...``.\n\nSupported options: ``passive_ports``\n\nDivergent string values:\n\n``passive_ports=&lt;min_port&gt;-&lt;max_port&gt;``\n</pre>', max_length=255, validators=[mini_buildd.models.FtpdOptionsValidator()]),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='gnupg_keyserver',
            field=models.CharField(default='keyserver.ubuntu.com', help_text='<pre>GnuPG keyserver to use (as fill-in helper)</pre>', max_length=200),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='gnupg_template',
            field=mini_buildd.models.UnixTextField(default='Key-Type: RSA\nKey-Length: 4096\nExpire-Date: 0', help_text='<pre>Template as accepted by &#x27;gpg --batch --gen-key&#x27; (see &#x27;man gpg&#x27;)\n\nYou should not give &#x27;Name-Real&#x27; and &#x27;Name-Email&#x27;, as these are automatically added.\n\nPrepare/Remove actions will generate/remove the GnuPG key.</pre>'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='identity',
            field=models.CharField(default=mini_buildd.config.default_identity, help_text='<pre>mini-buildd&#x27;s instance identity\n\nWill be used to identify this mini-buildd instance in various places:\n\n* &#x27;Name-Real&#x27; part of the GnuPG key.\n* name of the automated keyring packages.\n* the &#x27;Origin&#x27; tag of repository indices (but this may also be overridden via &#x27;custom_archive_origin&#x27; option below).\n* to identify remotes.\n</pre>', max_length=50, validators=[mini_buildd.models.RegexValidator(regex='^[a-zA-Z0-9\\-]+$')]),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='max_parallel_builds',
            field=models.IntegerField(default=mini_buildd.util.get_cpus, help_text='<pre>[Daemon must be restarted to activate this setting.]\n\nMaximum number of parallel builds</pre>'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='notify',
            field=models.ManyToManyField(blank=True, help_text='<pre>Addresses that get all notification emails unconditionally</pre>', to='mini_buildd.emailaddress'),
        ),
        migrations.AlterField(
            model_name='dirchroot',
            name='union_type',
            field=models.IntegerField(choices=[(0, 'aufs'), (1, 'overlayfs'), (2, 'unionfs'), (3, 'overlay')], default=3, help_text='<pre>See &#x27;man 5 schroot.conf&#x27;\n</pre>'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='add_depends',
            field=models.CharField(blank=True, help_text='<pre>Add extra (package) dependencies</pre>', max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='autopkgtest_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=0, help_text='<pre>Mode to control if a check should prevent package installation (for non-experimental suites).\n\n&#x27;DISABLED&#x27; (Don&#x27;t run check), &#x27;IGNORE&#x27; (Run check but ignore results), &#x27;ERRFAIL&#x27; (Run check and fail on errors), &#x27;WARNFAIL&#x27; (Run check and fail on warnings)</pre>'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='chroot_setup_script',
            field=mini_buildd.models.UnixTextField(blank=True, default='', help_text='<pre>Custom script that will be run via sbuild&#x27;s ``--chroot-setup-command``\n\nBe sure to provide a proper shebang like ``#!/bin/sh -e``.\n\nNote that some predefined script blocks to choose from are available (see &#x27;sbuild_setup_blocks&#x27;), which might suite your needs already.\n\nExample:\n---\n#!/bin/sh -e\n\necho &quot;I: Foo bar...&quot;\n---\n</pre>', validators=[mini_buildd.models.ShebangValidator()]),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='deb_build_options',
            field=models.CharField(blank=True, help_text='<pre>Set extra build options\n\nValues to add to environment variable ``DEB_BUILD_OPTIONS`` used when building packages. See\n\n* ``https://www.debian.org/doc/debian-policy/ch-source.html#debian-rules-and-deb-build-options`` and\n* ``https://lists.debian.org/debian-devel/2015/12/msg00262.html&#x27;&gt;Debian dbgsym announcement``\n\nfor valid options. Value may be overridden by the resp. user upload option.\n\nOption ``noddebs`` is useful for Ubuntu distributions that for\nsome reason create their automated debug packages with file appendix\n&#x27;ddeb&#x27;, as (see ``https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=730572``) reprepo fails on them.\n\nExample: Never build automatic debug packages (Ubuntu bionic, cosmic):\n\nnoddebs</pre>', max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='deb_build_profiles',
            field=models.CharField(blank=True, help_text='<pre>Set extra build profiles\n\nContents of DEB_BUILD_PROFILES environment for building (see ``https://wiki.debian.org/BuildProfileSpec``).\nValue may be overridden by the resp. user upload option.</pre>', max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='internal_apt_priority',
            field=models.IntegerField(default=1, help_text='<pre>Priority for internal APT sources in builds\n\nThe default is 1, which means you will only build against newer packages in our own repositories in case it&#x27;s really needed by the package&#x27;s build dependencies.This is the recommended behaviour, producing sparse dependencies.\n\nHowever, some packages with incorrect build dependencies might break anyway, while they would work fine when just build against the newest version available.\n\nYou may still solve this on a per-package basis, using the resp. upload option via changelog. However, in case you don&#x27;t care about sparse dependencies in this distribution in general, you can pimp the internal priority up here.\n\nExamples: 1=sparse dependencies, 500=always build against newer internal packages</pre>'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_extra_options',
            field=models.CharField(default='--info --suppress-tags bad-distribution-in-changes-file,bogus-mail-host,bogus-mail-host-in-debian-changelog', help_text='<pre>Lintian options to always use\n\nUseful examples (with hints in what lintian versions they are applicable):\n\n * &#x27;--info&#x27; (all): Show versatile explanations in buildlogs.\n * &#x27;--suppress-tags &lt;tag&gt;&#x27; (&gt;= 2.3.0, &gt;= Debian 6 squeeze): Suppress false-positive/too noisy tags.\n   * &#x27;bad-distribution-in-changes-file&#x27; (all): Lintian does not know mini-buildd-style distributions.\n   * &#x27;bogus-mail-host[-in-debian-changelog]&#x27; (&gt;= 2.104.0, &gt;= Debian 11 bullseye, &gt;= Ubuntu 21.10 impish): Not really mandatory for mini-buildd -- but lintian deems them errors, making lintian always fail.\n</pre>', max_length=512),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=2, help_text='<pre>Mode to control if a check should prevent package installation (for non-experimental suites).\n\n&#x27;DISABLED&#x27; (Don&#x27;t run check), &#x27;IGNORE&#x27; (Run check but ignore results), &#x27;ERRFAIL&#x27; (Run check and fail on errors), &#x27;WARNFAIL&#x27; (Run check and fail on warnings)</pre>'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_warnfail_options',
            field=models.CharField(default='--fail-on error,warning', help_text='<pre>Lintian options to use for &#x27;WARNFAIL&#x27; mode\n\nHumbly up-to-date variants of lintian all work with the default (&#x27;--fail-on error,warning&#x27;).\n\nYou may set exception here for some older releases if need be.\n\nFYI, lintian &#x27;warnfail&#x27; option history:\n * Debian (stretch)  2.5.50: Deprecates &#x27;--fail-on-warnings&#x27;.\n * Debian (bullseye) 2.16.0: Drops support for &#x27;--fail-on-warnings&#x27; (NO WAY TO DO &#x27;WARNFAIL&#x27;).\n * Debian (bullseye) 2.77.0: Introduces &#x27;--fail-on&#x27;.\n * Ubuntu (focal)    2.62.0: Release that does not have either (NO WAY TO DO &#x27;WARNFAIL&#x27;).\n * Ubuntu (groovy)   2.89.0: 1st release &gt;= 2.77.\nConclusion:\n * Debian &gt;=bullseye needs &#x27;--fail-on error,warning&#x27;, below needs &#x27;--fail-on-warnings&#x27;.\n * Ubuntu &gt;=groovy needs &#x27;--fail-on error,warning&#x27;, below needs &#x27;--fail-on-warnings&#x27;.\n * Ubuntu focal needs &#x27;&#x27; (just can&#x27;t do &#x27;WARNFAIL&#x27;).\n</pre>', max_length=512),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='piuparts_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=0, help_text='<pre>EXPERIMENTAL: Mode to control if a check should prevent package installation (for non-experimental suites).\n\n&#x27;DISABLED&#x27; (Don&#x27;t run check), &#x27;IGNORE&#x27; (Run check but ignore results), &#x27;ERRFAIL&#x27; (Run check and fail on errors), &#x27;WARNFAIL&#x27; (Run check and fail on warnings)\n\nEXPERIMENTAL: All modes but &#x27;Run check and ignore errors&#x27; are currently ignored, and even that might not be called correctly.\n</pre>'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuild_config_blocks',
            field=models.CharField(blank=True, help_text='<pre>Ordered sbuild config blocks (space-separated)\n\nDefault if not given: set-default-path ccache\n\nAvailable blocks:\n\nset-default-path: [default] Explicitly set $path (to sbuild&#x27;s default value), so later config blocks may add to it\nccache          : [default] Update $path and $$build environment to support ccache</pre>', max_length=255, validators=[mini_buildd.models.SbuildConfigBlocksValidator()]),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuild_setup_blocks',
            field=models.CharField(blank=True, help_text='<pre>Ordered sbuild setup blocks (space-separated)\n\nDefault if not given: apt-setup apt-update ccache eatmydata\n\nAvailable blocks:\n\napt-setup                : [default] Sets up APT for buildrequest (APT keys, optional https, sources and preferences)\napt-update               : [default] Update APT\nccache                   : [default] Installs ccache (together with the &#x27;ccache&#x27; config block, this enables ccache for builds on the same instance)\neatmydata                : [default] Install eatmydata and activate via LD_PRELOAD when possible (may improve speed)\nshow-sbuildrc            : [extra] Print generated &#x27;.sbuildrc&#x27; (informational only)\napt-key-binary           : [extra] Convert APT keys to binary (needed for Debian &lt;= jessie, Ubuntu &lt;= yakketti) -- place between ``apt-setup`` and ``apt-update``\napt-key-add              : [extra] Add APT keys via deprecated ``apt-key add`` (needed for Debian &lt;= squeeze) -- place between ``apt-setup`` and ``apt-update``\napt-no-check-valid-until : [extra] Set APT option &quot;Acquire::Check-Valid-Until false&quot; (needed when Release files are expired) -- place before ``apt-setup``\napt-allow-unauthenticated: [extra] Set APT option &quot;APT::Get::AllowUnauthenticated true&quot; (needed when APT keys are expired) -- place before ``apt-setup``\napt-force-yes            : [extra] Set APT option &quot;APT::Get::Force-Yes true&quot; (compat(urold)) -- place before ``apt-setup``\napt-no-cache             : [extra] Set APT option &quot;Acquire::http::No-Cache true&quot; (compat(urold)) -- place before ``apt-setup``\napt-no-pipelining        : [extra] Set APT option &quot;Acquire::http::Pipeline-Depth 0&quot; (compat(urold)) -- place before ``apt-setup``\napt-clear                : [extra] Try to clear any redundant APT data in ``/var/`` (experimental: can make APT work again if stuck)\nbash-devices             : [extra] Fixup /dev/fd|stdin|stdout|stderr. Needed for *building* bash only. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=327477 (compat(bash &lt; 3.0-17 (2005))\nschroot-shm              : [extra] Fixup /dev/shm mount. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=728096 (compat(schroot &lt; 1.6.10-3))\nsun-java6-license        : [extra] Accept sun-java6 licence (debconf) so we can build-depend on it (sun-java is no longer part of Debian since 2011)</pre>', max_length=255, validators=[mini_buildd.models.SbuildSetupBlocksValidator()]),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuildrc_snippet',
            field=mini_buildd.models.UnixTextField(blank=True, default='', help_text='<pre>Custom perl snippet to be added to ``.sbuildrc`` for each build\n\nNote that some predefined config blocks to choose from are available (see &#x27;sbuild_config_blocks&#x27;), which might suite your needs already.\n\nYou may use these placeholders:\n\n* ``%LIBDIR%``: Per-chroot persistent dir; may be used for data that should persist between builds (caches, ...).\n\nExample:\n---\n$build_environment = { &#x27;MY_FOO_BAR&#x27; =&gt; &#x27;%LIBDIR%/my-foo-bar&#x27; };\n---\n</pre>'),
        ),
        migrations.AlterField(
            model_name='layout',
            name='default_version',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%+1', help_text='<pre>Version string to append to the original version for automated ports\n\nYou may use these placeholders:\n* %IDENTITY%: Repository identity (see &#x27;Repository&#x27;)\n* %CODEVERSION%: Numerical base distribution version (see &#x27;Source&#x27;)</pre>', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='experimental_default_version',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%+0', help_text='<pre>Like &#x27;default version&#x27;, but for suites flagged &#x27;experimental&#x27;</pre>', max_length=30),
        ),
        migrations.AlterField(
            model_name='layout',
            name='experimental_mandatory_version_regex',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%\\+0', help_text='<pre>Like &#x27;mandatory version&#x27;, but for suites flagged &#x27;experimental&#x27;</pre>', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='mandatory_version_regex',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%\\+[1-9]', help_text='<pre>Mandatory version regex (you may use the same placeholders as for &#x27;default version&#x27;)</pre>', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='meta_distributions',
            field=models.JSONField(blank=True, default=dict, help_text='<pre>Support METAs alone as distribution identifier\n\nMeta distribution identifiers should be unique across all repositories; usually, a layout with meta distributions should only be used by at most one repository.\n\nExample:\n\n{&#x27;unstable&#x27;: &#x27;sid-unstable&#x27;, &#x27;experimental&#x27;=&#x27;sid-experimental&#x27;}\n\n(See standard layout &#x27;Debian Developer&#x27;): allows upload/testing of packages (to unstable,experimental,..) aimed for Debian.</pre>'),
        ),
        migrations.AlterField(
            model_name='looplvmchroot',
            name='loop_size',
            field=models.IntegerField(default=100, help_text='<pre>Loop device file size in GB</pre>'),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='snapshot_size',
            field=models.IntegerField(default=4, help_text='<pre>Snapshot device file size in GB</pre>'),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='volume_group',
            field=models.CharField(default='auto', help_text='<pre>Give a pre-existing LVM volume group name. Just leave it on &#x27;auto&#x27; for loop lvm chroots.</pre>', max_length=80),
        ),
        migrations.AlterField(
            model_name='prioritysource',
            name='priority',
            field=models.IntegerField(default=1, help_text='<pre>A APT pin priority value (see &#x27;man apt_preferences&#x27;; examples: 1=not automatic, 1001=downgrade)</pre>'),
        ),
        migrations.AlterField(
            model_name='remote',
            name='http',
            field=models.CharField(default='tcp:host=YOUR_REMOTE_HOST:port=8066', help_text='<pre>HTTP Remote Endpoint: Twisted-style Network Client Endpoint\n\nSee: https://twistedmatrix.com/documents/current/core/howto/endpoints.html#clients\n\nNotation Examples::\n\n  &#x27;tcp:host=localhost:port=8066&#x27;: Connect to localhost, unencrypted.\n  &#x27;tls:host=localhost:port=8066&#x27;: Connect to localhost, encrypted.\n  &#x27;http://localhost:8066&#x27;       : Extra URL-style syntax (prefer twisted-style in saved configs, though).\n  &#x27;localhost:8066&#x27;              : Compatibility host:port syntax (don&#x27;t use -- might eventually go away).</pre>', max_length=255, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default='', help_text='<pre>Actual (ASCII-armored) GnuPG public key (leave ``key_id`` above blank)</pre>'),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_created',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_expires',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_id',
            field=models.CharField(blank=True, default='', help_text='<pre>Key id to retrieve the actual key per key server (leave ``key`` below blank)</pre>', max_length=100, validators=[django.core.validators.MinLengthValidator(8, message='The key id, if given, must be at least 8 bytes long')]),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_long_id',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_name',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='wake_command',
            field=models.CharField(blank=True, default='', help_text='<pre>Command to wake this remote (for example ``/usr/bin/wakeonlan -i DEST_IP MAC``)</pre>', max_length=255),
        ),
        migrations.AlterField(
            model_name='repository',
            name='allow_unauthenticated_uploads',
            field=models.BooleanField(default=False, help_text='<pre>Allow unauthenticated user uploads</pre>'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='extra_uploader_keyrings',
            field=mini_buildd.models.UnixTextField(blank=True, help_text='<pre>Extra keyrings, line by line, to be allowed as uploaders (in addition to configured django users)\n\nExample:\n---\n# Allow Debian maintainers (must install the &#x27;debian-keyring&#x27; package)\n/usr/share/keyrings/debian-keyring.gpg\n# Allow from some local keyring file\n/etc/my-schlingels.gpg\n---</pre>'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='identity',
            field=models.CharField(default='test', help_text='<pre>The id of the reprepro repository, placed in &#x27;repositories/&lt;ID&gt;&#x27;\n\nIt can also be used in &#x27;version enforcement string&#x27; (true for the default layout) -- in this context, it plays the same role as the well-known &#x27;bpo&#x27; version string from Debian backports.</pre>', max_length=50, primary_key=True, serialize=False, validators=[mini_buildd.models.RegexValidator(regex='^[a-z0-9]+$')]),
        ),
        migrations.AlterField(
            model_name='repository',
            name='notify',
            field=models.ManyToManyField(blank=True, help_text='<pre>Addresses that get all notification emails unconditionally</pre>', to='mini_buildd.emailaddress'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='notify_changed_by',
            field=models.BooleanField(default=False, help_text='<pre>Notify the address in the &#x27;Changed-By&#x27; field of the uploaded changes file</pre>'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='notify_maintainer',
            field=models.BooleanField(default=False, help_text='<pre>Notify the address in the &#x27;Maintainer&#x27; field of the uploaded changes file</pre>'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='reprepro_morguedir',
            field=models.BooleanField(default=False, help_text='<pre>Move files deleted from repo pool to &#x27;morguedir&#x27; (see reprepro)</pre>'),
        ),
        migrations.AlterField(
            model_name='source',
            name='apt_keys',
            field=models.ManyToManyField(blank=True, help_text='<pre>At least one APT key this source&#x27;s &#x27;Release&#x27; file is signed with\n\nHint: If you leave this empty and run &#x27;check&#x27; on it, you will get a message listing all signer keys.</pre>', to='mini_buildd.aptkey'),
        ),
        migrations.AlterField(
            model_name='source',
            name='architectures',
            field=models.ManyToManyField(blank=True, editable=False, to='mini_buildd.architecture'),
        ),
        migrations.AlterField(
            model_name='source',
            name='archives',
            field=models.ManyToManyField(blank=True, editable=False, to='mini_buildd.archive'),
        ),
        migrations.AlterField(
            model_name='source',
            name='check_valid_until',
            field=models.BooleanField(default=True, help_text='<pre>Fail if outdated (via ``Valid-Until`` field)\n\nSome sources have a ``Valid-Until`` field that is no longer updated. Disable this if you really still want to use it anyway.\n\nWhen disabled, this will 1st ignore mini-buildd&#x27;s own &#x27;Valid-Until check&#x27; and 2nd, create APT lines for this source with the ``[check-valid-until=no]`` option.\n\nNote: For ``jessie`` or older (where this APT line option does not work), ``apt-no-check-valid-until`` sbuild setup block is still needed.</pre>'),
        ),
        migrations.AlterField(
            model_name='source',
            name='codename',
            field=models.CharField(default='sid', help_text='<pre>The name of the directory below ``dists/`` in archives this source refers to (this is also the 3rd part of an APT line)\n\nAlso the exact string of the ``Codename`` field of the resp, Release file (unless overwritten in ``extra_identifiers``).</pre>', max_length=60),
        ),
        migrations.AlterField(
            model_name='source',
            name='codeversion',
            field=models.CharField(blank=True, default='', editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name='source',
            name='codeversion_override',
            field=models.CharField(blank=True, default='', help_text='<pre>Permanently override with a static ``codeversion`` value\n\nLeave blank to make ``codeversion`` be guessed every time the source is checked.\n\nThe ``codeversion`` (like &#x27;12&#x27; for ``Debian bookworm``) is relevant for base sources only, to be used for version restrictions.</pre>', max_length=50),
        ),
        migrations.AlterField(
            model_name='source',
            name='components',
            field=models.ManyToManyField(blank=True, editable=False, to='mini_buildd.component'),
        ),
        migrations.AlterField(
            model_name='source',
            name='extra_identifiers',
            field=models.JSONField(blank=True, default=dict, help_text='<pre>Extra release file identifiers (JSON dict)\n\nSome sources need additional fields (same as in a Release file) to further specify this source. These are also later used to pin the source via APT.\n\nMost likely candidates are ``Codename``, ``Suite``, ``Archive``, ``Version`` or ``Label``.\n\nFor some sources, ``Codename`` (as we use it above) does not match resp. value as given in the Release file.\nWhen ``Codename`` is overridden in this manner, be sure to also add one further flag to identify the source -- else apt pinning later would likely not be unambiguous.\nReal world examples that need this extra handling are ``Debian Security`` (``buster`` or older), ``Ubuntu Security and Backports``:\n\n{&#x27;Codename&#x27;: &#x27;stretch&#x27;, &#x27;Label&#x27;: &#x27;Debian-Security&#x27;}\n{&#x27;Codename&#x27;: &#x27;bionic&#x27;, &#x27;Suite&#x27;: &#x27;bionic-backports&#x27;}</pre>', validators=[mini_buildd.models.ForbiddenKeysValidator(['Origin'])]),
        ),
        migrations.AlterField(
            model_name='source',
            name='origin',
            field=models.CharField(default='Debian', help_text='<pre>The exact string of the &#x27;Origin&#x27; field of the resp. Release file</pre>', max_length=60),
        ),
        migrations.AlterField(
            model_name='source',
            name='remove_from_component',
            field=models.CharField(blank=True, default='', help_text='<pre>Prefix to be removed from component\n\nSome (actually, we only know of ``Debian Security``) sources have weird ``Components`` that need to be fixed to work with mini-buildd.\n\nFor example, ``Debian Security`` needs ``updates/``.</pre>', max_length=100),
        ),
        migrations.AlterField(
            model_name='suite',
            name='name',
            field=models.CharField(help_text='<pre>A suite name (usually s.th. like &#x27;experimental&#x27;, &#x27;unstable&#x27;, &#x27;testing&#x27; or &#x27;stable&#x27;)</pre>', max_length=100, validators=[mini_buildd.models.RegexValidator(regex='^[a-z]+$')]),
        ),
        migrations.AlterField(
            model_name='suiteoption',
            name='build_keyring_package',
            field=models.BooleanField(default=False, help_text='<pre>Build keyring package for this suite (i.e., when the resp. Repository action is called)</pre>'),
        ),
        migrations.AlterField(
            model_name='suiteoption',
            name='but_automatic_upgrades',
            field=models.BooleanField(default=True, help_text='<pre>Include &#x27;ButAutomaticUpgrades&#x27; in the Release file</pre>'),
        ),
        migrations.AlterField(
            model_name='suiteoption',
            name='experimental',
            field=models.BooleanField(default=False, help_text='<pre>Whether this suite is flagged experimental\n\nExperimental suites must be uploadable and must not migrate.\nAlso, the packager treats failing extra QA checks (like lintian) as non-lethal, and will install anyway.</pre>'),
        ),
        migrations.AlterField(
            model_name='suiteoption',
            name='migrates_to',
            field=models.ForeignKey(blank=True, help_text='<pre>Give another suite where packages may migrate to (you may need to save this &#x27;blank&#x27; first before you see choices here)</pre>', null=True, on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.suiteoption'),
        ),
        migrations.AlterField(
            model_name='suiteoption',
            name='not_automatic',
            field=models.BooleanField(default=True, help_text='<pre>Include &#x27;NotAutomatic&#x27; in the Release file</pre>'),
        ),
        migrations.AlterField(
            model_name='suiteoption',
            name='rollback',
            field=models.IntegerField(default=0, help_text='<pre>Keep this number of old versions in this suite</pre>'),
        ),
        migrations.AlterField(
            model_name='suiteoption',
            name='uploadable',
            field=models.BooleanField(default=True, help_text='<pre>Whether this suite should accept user uploads</pre>'),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default='', help_text='<pre>Actual (ASCII-armored) GnuPG public key (leave ``key_id`` above blank)</pre>'),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_created',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_expires',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_id',
            field=models.CharField(blank=True, default='', help_text='<pre>Key id to retrieve the actual key per key server (leave ``key`` below blank)</pre>', max_length=100, validators=[django.core.validators.MinLengthValidator(8, message='The key id, if given, must be at least 8 bytes long')]),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_long_id',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_name',
            field=models.CharField(blank=True, default='', editable=False, max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='user',
            field=models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 4.2.9 on 2024-01-06 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0012_migrate_layout_extra_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='layout',
            name='meta_distributions',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='source',
            name='extra_identifiers',
            field=models.JSONField(blank=True, default=dict),
        ),
    ]



"""
By django convention this module is used to register models for the admin site

See: https://docs.djangoproject.com/en/3.1/ref/contrib/admin/#django.contrib.admin.autodiscover
"""
import django.contrib

from mini_buildd import models

MODELS = [
    models.AptKey,
    models.Uploader,
    models.Remote,
    models.Archive,
    models.Architecture,
    models.Component,
    models.Source,
    models.PrioritySource,
    models.EmailAddress,
    models.Suite,
    models.Layout,
    models.Distribution,
    models.Repository,
    models.Chroot,
    models.DirChroot,
    models.FileChroot,
    models.LVMChroot,
    models.LoopLVMChroot,
    models.BtrfsSnapshotChroot,
    models.Daemon,
    models.Subscription,
]


class AdminSite(django.contrib.admin.sites.AdminSite):
    index_template = "admin/mini_buildd.html"

    __APP_DATA = {
        "mini_buildd": {"order": 1, "setup": "setup"},
        "auth": {"order": 2},
    }
    __MODEL_DATA = {
        # Daemon
        "Daemon": {
            "class_path": "Daemon",
            "header": "Daemon",
            "order": 0,
        },
        # Sources
        "Archive": {
            "class_path": "Archive",
            "header": "Sources",
            "order": 10,
        },
        "Source": {
            "class_path": "Source",
            "order": 11,
        },
        "PrioritySource": {
            "class_path": "PrioritySource",
            "order": 12,
        },
        "AptKey": {
            "class_path": "AptKey",
            "order": 13,
        },
        # Repositories
        "Layout": {
            "class_path": "Layout",
            "header": "Repositories",
            "order": 20,
        },
        "Distribution": {
            "class_path": "Distribution",
            "order": 21,
        },
        "Repository": {
            "class_path": "Repository",
            "order": 22,
        },
        "Uploader": {
            "class_path": "Uploader",
            "order": 23,
        },
        # Chroots
        "DirChroot": {
            "class_path": "DirChroot",
            "header": "Chroots",
            "order": 30,
        },
        "FileChroot": {
            "class_path": "FileChroot",
            "order": 31,
        },
        "LVMChroot": {
            "class_path": "LVMChroot",
            "order": 32,
        },
        "LoopLVMChroot": {
            "class_path": "LoopLVMChroot",
            "order": 33,
        },
        "BtrfsSnapshotChroot": {
            "class_path": "BtrfsSnapshotChroot",
            "order": 34,
        },
        # Remotes
        "Remote": {
            "class_path": "Remote",
            "header": "Remotes",
            "order": 40,
        },
        # Others
        "Architecture": {
            "class_path": "Architecture",
            "header": "Others",
            "order": 50,
        },
        "Component": {
            "class_path": "Component",
            "order": 51,
        },
        "EmailAddress": {
            "class_path": "EmailAddress",
            "order": 52,
        },
        "Suite": {
            "class_path": "Suite",
            "order": 53,
        },
        "Subscription": {
            "class_path": "Subscription",
            "order": 54,
        },
        "Chroot": {
            "class_path": "Chroot",
            "order": 55,
        },
    }

    def __init__(self):
        super().__init__("mini-buildd-admin")
        # Default action 'delete_selected' action does not call
        # custom delete.
        #
        # See: https://docs.djangoproject.com/en/dev/ref/contrib/admin/actions/
        #
        # So we just disable this default action. You can still delete
        # single objects from the model's form.
        self.disable_action("delete_selected")

        for m in MODELS:
            admin_class = getattr(m, "Admin")
            self.register(m, admin_class)
            self._registry.update(django.contrib.admin.site._registry)  # Add other apps that have been autodiscovered

    def get_app_list(self, request, app_label=None):  # pylint: disable=unused-argument  # Needed for django < 4.1 -- 4.1 added 'app_label=None'
        """Proper order && group header for mini_buildd app; others apps still sorted alphabetically"""
        app_dict = self._build_app_dict(request)
        app_list = sorted(app_dict.values(), key=lambda app: self.__APP_DATA[app["app_label"]].get("order", 999))
        for app in app_list:
            if app["app_label"] == "mini_buildd":
                app["models"].sort(key=lambda model: self.__MODEL_DATA[model["object_name"]].get("order", 999))
                app["mbd"] = self.__APP_DATA["mini_buildd"]

                for model in app["models"]:
                    model["mbd"] = self.__MODEL_DATA[model["object_name"]]
            else:
                app['models'].sort(key=lambda model: model['name'])

        return app_list


site = AdminSite()



# Generated by Django 4.2.9 on 2024-01-06 14:14

from django.db import migrations, models
import mini_buildd.models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0008_remove_daemon_hostname'),
    ]

    operations = [
        migrations.AddField(
            model_name='source',
            name='check_valid_until',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='source',
            name='extra_identifiers',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='source',
            name='remove_from_component',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='source',
            name='extra_options',
            field=mini_buildd.models.UnixTextField(blank=True),
        ),
    ]



# Generated by Django 4.2.8 on 2024-01-03 16:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0003_remove_daemon_sbuild_jobs'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='distribution',
            name='apt_allow_unauthenticated',
        ),
    ]



"""
Extra, independent code, merely arbitrarily stuffed here

This is not part of mini-buildd.

"""

import urllib.request

import bs4


class DebianPackageTracker():
    """
    Get (some) source package information from the Debian Package Tracker

    As long as there is no proper API, we do this the
    hacky way parsing the HTML via bs4 - so this will most
    definitely break at some point. Use at own discretion.
    """

    def __init__(self, src_package, tracker_url="https://tracker.debian.org"):
        self.info = {}

        pkg_url = f"{tracker_url}/pkg/{src_package}"
        with urllib.request.urlopen(pkg_url) as response:
            soup = bs4.BeautifulSoup(response.read(), features="html.parser")
            version_tags = soup.findAll("span", {"class": "versions-repository"})
            for d in version_tags:
                codename = d['title'].split("(")[1].split(" ")[0].translate({ord(")"): None})
                version = d.find_next_sibling("a").get_text("", strip=True)
                version_url = version.replace("~", "").replace("+", "")
                self.info[codename] = {
                    "version": version,
                    "changelog_url": f"{tracker_url}/media/packages/{src_package[:4] if src_package.startswith('lib') else src_package[0]}/{src_package}/changelog-{version_url}"
                }

    def get_info(self, codename):
        return self.info[codename]

    def get_version(self, codename):
        return self.info[codename]["version"]



# Generated by Django 4.2.8 on 2024-01-03 15:23

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import mini_buildd.util
import mini_buildd.models
import mini_buildd.config


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AptKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('status', models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False)),
                ('last_checked', models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False)),
                ('auto_reactivate', models.BooleanField(default=False, editable=False)),
                ('key_id', models.CharField(blank=True, default='', max_length=100)),
                ('key', mini_buildd.models.UnixTextField(blank=True, default='')),
                ('key_long_id', models.CharField(blank=True, default='', max_length=254)),
                ('key_created', models.CharField(blank=True, default='', max_length=254)),
                ('key_expires', models.CharField(blank=True, default='', max_length=254)),
                ('key_name', models.CharField(blank=True, default='', max_length=254)),
                ('key_fingerprint', models.CharField(blank=True, default='', max_length=254)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Architecture',
            fields=[
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('name', models.CharField(max_length=50, primary_key=True, serialize=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ArchitectureOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('optional', models.BooleanField(default=False)),
                ('build_architecture_all', models.BooleanField(default=False)),
                ('architecture', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.architecture')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Archive',
            fields=[
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('url', models.URLField(default='http://ftp.debian.org/debian/', max_length=512, primary_key=True, serialize=False)),
                ('ping', models.FloatField(default=-1.0, editable=False)),
            ],
            options={
                'ordering': ['url'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Chroot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('status', models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False)),
                ('last_checked', models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False)),
                ('auto_reactivate', models.BooleanField(default=False, editable=False)),
                ('personality', models.CharField(blank=True, default='', editable=False, max_length=50)),
                ('personality_override', models.CharField(blank=True, default='', max_length=50)),
                ('architecture', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.architecture')),
            ],
            options={
                'ordering': ['source', 'architecture'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Component',
            fields=[
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('name', models.CharField(max_length=50, primary_key=True, serialize=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Distribution',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('build_dep_resolver', models.IntegerField(choices=[(0, 'apt'), (1, 'aptitude'), (2, 'internal')], default=1)),
                ('apt_allow_unauthenticated', models.BooleanField(default=False)),
                ('lintian_mode', models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=2)),
                ('lintian_extra_options', models.CharField(default='--info', max_length=200)),
                ('piuparts_mode', models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=0)),
                ('piuparts_extra_options', models.CharField(default='--info', max_length=200)),
                ('piuparts_root_arg', models.CharField(default='sudo', max_length=200)),
                ('chroot_setup_script', mini_buildd.models.UnixTextField(blank=True, default='')),
                ('sbuildrc_snippet', mini_buildd.models.UnixTextField(blank=True, default='')),
                ('architectures', models.ManyToManyField(through='mini_buildd.ArchitectureOption', to='mini_buildd.architecture')),
            ],
            options={
                'ordering': ['base_source'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EmailAddress',
            fields=[
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('address', models.EmailField(max_length=255, primary_key=True, serialize=False)),
                ('name', models.CharField(blank=True, max_length=255)),
            ],
            options={
                'verbose_name_plural': 'Email addresses',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Layout',
            fields=[
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, primary_key=True, serialize=False)),
                ('default_version', models.CharField(default='~%IDENTITY%%CODEVERSION%+1', max_length=100)),
                ('mandatory_version_regex', models.CharField(default='~%IDENTITY%%CODEVERSION%\\+[1-9]', max_length=100)),
                ('experimental_default_version', models.CharField(default='~%IDENTITY%%CODEVERSION%+0', max_length=30)),
                ('experimental_mandatory_version_regex', models.CharField(default='~%IDENTITY%%CODEVERSION%\\+0', max_length=100)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Remote',
            fields=[
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('status', models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False)),
                ('last_checked', models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False)),
                ('auto_reactivate', models.BooleanField(default=False, editable=False)),
                ('key_id', models.CharField(blank=True, default='', max_length=100)),
                ('key', mini_buildd.models.UnixTextField(blank=True, default='')),
                ('key_long_id', models.CharField(blank=True, default='', max_length=254)),
                ('key_created', models.CharField(blank=True, default='', max_length=254)),
                ('key_expires', models.CharField(blank=True, default='', max_length=254)),
                ('key_name', models.CharField(blank=True, default='', max_length=254)),
                ('key_fingerprint', models.CharField(blank=True, default='', max_length=254)),
                ('http', models.CharField(default='tcp:host=YOUR_REMOTE_HOST:port=8066', max_length=255, primary_key=True, serialize=False)),
                ('wake_command', models.CharField(blank=True, default='', max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Repository',
            fields=[
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('status', models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False)),
                ('last_checked', models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False)),
                ('auto_reactivate', models.BooleanField(default=False, editable=False)),
                ('identity', models.CharField(default='test', max_length=50, primary_key=True, serialize=False)),
                ('allow_unauthenticated_uploads', models.BooleanField(default=False)),
                ('extra_uploader_keyrings', mini_buildd.models.UnixTextField(blank=True)),
                ('notify_changed_by', models.BooleanField(default=False)),
                ('notify_maintainer', models.BooleanField(default=False)),
                ('reprepro_morguedir', models.BooleanField(default=False)),
                ('external_home_url', models.URLField(blank=True)),
                ('distributions', models.ManyToManyField(to='mini_buildd.distribution')),
                ('layout', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.layout')),
                ('notify', models.ManyToManyField(blank=True, to='mini_buildd.emailaddress')),
            ],
            options={
                'verbose_name_plural': 'Repositories',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Suite',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BtrfsSnapshotChroot',
            fields=[
                ('chroot_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot')),
            ],
            options={
                'abstract': False,
            },
            bases=('mini_buildd.chroot',),
        ),
        migrations.CreateModel(
            name='DirChroot',
            fields=[
                ('chroot_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot')),
                ('union_type', models.IntegerField(choices=[(0, 'aufs'), (1, 'overlayfs'), (2, 'unionfs'), (3, 'overlay')], default=3)),
            ],
            options={
                'abstract': False,
            },
            bases=('mini_buildd.chroot',),
        ),
        migrations.CreateModel(
            name='FileChroot',
            fields=[
                ('chroot_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot')),
                ('compression', models.IntegerField(choices=[(0, 'no compression'), (1, 'gzip'), (2, 'bzip2'), (3, 'xz')], default=0)),
            ],
            options={
                'abstract': False,
            },
            bases=('mini_buildd.chroot',),
        ),
        migrations.CreateModel(
            name='LVMChroot',
            fields=[
                ('chroot_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot')),
                ('volume_group', models.CharField(default='auto', max_length=80)),
                ('filesystem', models.CharField(default='ext2', max_length=10)),
                ('snapshot_size', models.IntegerField(default=4)),
            ],
            options={
                'abstract': False,
            },
            bases=('mini_buildd.chroot',),
        ),
        migrations.CreateModel(
            name='Uploader',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('status', models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False)),
                ('last_checked', models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False)),
                ('auto_reactivate', models.BooleanField(default=False, editable=False)),
                ('key_id', models.CharField(blank=True, default='', max_length=100)),
                ('key', mini_buildd.models.UnixTextField(blank=True, default='')),
                ('key_long_id', models.CharField(blank=True, default='', max_length=254)),
                ('key_created', models.CharField(blank=True, default='', max_length=254)),
                ('key_expires', models.CharField(blank=True, default='', max_length=254)),
                ('key_name', models.CharField(blank=True, default='', max_length=254)),
                ('key_fingerprint', models.CharField(blank=True, default='', max_length=254)),
                ('may_upload_to', models.ManyToManyField(blank=True, to='mini_buildd.repository')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SuiteOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('uploadable', models.BooleanField(default=True)),
                ('experimental', models.BooleanField(default=False)),
                ('build_keyring_package', models.BooleanField(default=False)),
                ('auto_migrate_after', models.IntegerField(default=0)),
                ('not_automatic', models.BooleanField(default=True)),
                ('but_automatic_upgrades', models.BooleanField(default=True)),
                ('layout', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.layout')),
                ('migrates_to', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.suiteoption')),
                ('suite', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.suite')),
            ],
            options={
                'abstract': False,
                'unique_together': {('suite', 'layout')},
            },
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('package', models.CharField(blank=True, max_length=100)),
                ('distribution', models.CharField(blank=True, max_length=100)),
                ('subscriber', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Source',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('status', models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False)),
                ('last_checked', models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False)),
                ('auto_reactivate', models.BooleanField(default=False, editable=False)),
                ('origin', models.CharField(default='Debian', max_length=60)),
                ('codename', models.CharField(default='sid', max_length=60)),
                ('description', models.CharField(blank=True, default='', editable=False, max_length=100)),
                ('codeversion', models.CharField(blank=True, default='', max_length=50)),
                ('codeversion_override', models.CharField(blank=True, default='', editable=False, max_length=50)),
                ('apt_keys', models.ManyToManyField(blank=True, to='mini_buildd.aptkey')),
                ('architectures', models.ManyToManyField(blank=True, to='mini_buildd.architecture')),
                ('archives', models.ManyToManyField(blank=True, to='mini_buildd.archive')),
                ('components', models.ManyToManyField(blank=True, to='mini_buildd.component')),
            ],
            options={
                'ordering': ['origin', '-codeversion', 'codename'],
                'abstract': False,
                'unique_together': {('origin', 'codename')},
            },
        ),
        migrations.CreateModel(
            name='PrioritySource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('priority', models.IntegerField(default=1)),
                ('source', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source')),
            ],
            options={
                'abstract': False,
                'unique_together': {('source', 'priority')},
            },
        ),
        migrations.AddField(
            model_name='layout',
            name='suites',
            field=models.ManyToManyField(through='mini_buildd.SuiteOption', to='mini_buildd.suite'),
        ),
        migrations.AddField(
            model_name='distribution',
            name='base_source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AddField(
            model_name='distribution',
            name='components',
            field=models.ManyToManyField(to='mini_buildd.component'),
        ),
        migrations.AddField(
            model_name='distribution',
            name='extra_sources',
            field=models.ManyToManyField(blank=True, to='mini_buildd.prioritysource'),
        ),
        migrations.CreateModel(
            name='Daemon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_options', mini_buildd.models.UnixTextField(blank=True)),
                ('pickled_data', models.TextField(blank=True)),
                ('status', models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False)),
                ('last_checked', models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False)),
                ('auto_reactivate', models.BooleanField(default=False, editable=False)),
                ('identity', models.CharField(default=mini_buildd.config.default_identity, max_length=50)),
                ('hostname', models.CharField(blank=True, max_length=200)),
                ('email_address', mini_buildd.models.DaemonEmailField(default=mini_buildd.config.default_email_address, max_length=255)),
                ('gnupg_template', mini_buildd.models.UnixTextField(default='Key-Type: RSA\nKey-Length: 4096\nExpire-Date: 0')),
                ('gnupg_keyserver', models.CharField(default='keyserver.ubuntu.com', max_length=200)),
                ('ftpd_bind', models.CharField(default='tcp:port=8067', max_length=200)),
                ('ftpd_options', models.CharField(blank=True, default='', max_length=255)),
                ('build_queue_size', models.IntegerField(default=mini_buildd.util.get_cpus)),
                ('sbuild_jobs', models.IntegerField(default=1)),
                ('smtp_server', models.CharField(default='localhost:25', max_length=254)),
                ('allow_emails_to', models.CharField(default=mini_buildd.config.default_allow_emails_to, max_length=254)),
                ('custom_hooks_directory', models.CharField(blank=True, default='', max_length=255)),
                ('show_last_packages', models.IntegerField(default=500)),
                ('show_last_builds', models.IntegerField(default=100)),
                ('wait_for_build_results', models.IntegerField(default=5)),
                ('keep_build_results', models.IntegerField(default=5)),
                ('notify', models.ManyToManyField(blank=True, to='mini_buildd.emailaddress')),
            ],
            options={
                'verbose_name_plural': 'Daemon',
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='chroot',
            name='source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AddField(
            model_name='architectureoption',
            name='distribution',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.distribution'),
        ),
        migrations.CreateModel(
            name='LoopLVMChroot',
            fields=[
                ('lvmchroot_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.lvmchroot')),
                ('loop_size', models.IntegerField(default=100)),
            ],
            options={
                'abstract': False,
            },
            bases=('mini_buildd.lvmchroot',),
        ),
        migrations.AlterUniqueTogether(
            name='chroot',
            unique_together={('source', 'architecture')},
        ),
        migrations.AlterUniqueTogether(
            name='architectureoption',
            unique_together={('architecture', 'distribution')},
        ),
    ]



# Generated by Django 4.2.8 on 2024-01-03 16:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0005_remove_daemon_smtp_server'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='daemon',
            name='custom_hooks_directory',
        ),
    ]



"""
.. attention:: **sbuild** (retry-workaround in ``gnupg.py``): Spurious build failures ``can't connect to the agent: IPC connect call failed`` (:debbug:`849551`)

.. attention:: **sbuild**: Fails to build "all" packages with "build archall" flag set to arch "x" in case DSP has >= 1 arch "all" and >=1 arch "y" binary package (:debbug:`706086`)

  This is due to sbuild and in in more detail explained in the Debian bug report.

  A bad one-package workaround would be to change the "build archall" flag to arch "y".

.. attention:: **sbuild** (workaround in ``apt-update`` snippet): ``apt-get update`` "File has unexpected size ... Mirror sync in progress?" failures

  This may happen if by chance there are reprepro updates around the same time ``apt-get update`` runs in the build.

  You may reproduce this quite reliably with the testsuite (it's due to the immediate parent package remove, see ``./devel``):

  1. testsuite: Change ``installed-port`` test to (also) port for running codename (``changelog`` + ``./devel``)
  2. ./devel updatetestall
  3. ./devel testsuite-packages installed-port

  Internal sbuild snippet works around this problem by retrying ``apt-get update`` -- but you still see the error lines
  in the buildlog if that happens.

  Solution could be to lock HTTP delivery on any reprepro change, but for now this seems overkill to me.

.. attention:: **sbuild+apt-cacher-ng** (workaround in ``apt-update`` snippet): ``apt-get update`` stalls on parallel access

  **Phenomena**: A build just always remains ongoing && the buildlog just stalls during ``apt-get update``.

  **Repeat bug**:

  .. literalinclude:: ../../examples/mini-buildd/apt-parallel-bug
    :caption: /usr/share/doc/mini-buildd/examples/apt-parallel-bug

  **Remedy situation at hand**:

  1. Kill corresponding process ``/usr/lib/apt/methods/gpgv`` or ``/usr/lib/apt/methods/http`` (apt-get update may be retried and build continues)
  2. Cancel resp. build(s) (then retry)

  **Related 3rd party findings**:

    * https://bugs.launchpad.net/ubuntu/+source/apt-cacher-ng/+bug/1983856
    * https://bugs.launchpad.net/ubuntu/+source/apt/+bug/2003851
"""
import logging
import os
import re
import subprocess
import time

from mini_buildd import call, config, dist, files, util

LOG = logging.getLogger(__name__)


#: Build dir constants.
CONFIG_DIR = ".config"
CONFIG_APT_KEYS = "apt_keys"
CONFIG_APT_PREFERENCES = "apt_preferences"
CONFIG_APT_SOURCES_LIST = "apt_sources.list"
CONFIG_CHROOT_SETUP_SCRIPT = "chroot_setup_script"
CONFIG_SBUILDRC_SNIPPET = "sbuildrc_snippet"
CONFIG_SSL_CERT = "ssl_cert"

SETUP_D_DIR = ".setup.d"

#: Quiet, non-interactive, least invasive and loggable apt-get call (Dpkg::Use-Pty=false is to avoid https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=539617).
APT_GET = "apt-get --quiet --yes --option=APT::Install-Recommends=false --option=Acquire::Languages=none --option=Dpkg::Use-Pty=false --option=Dpkg::Options::=--force-confdef --option=Dpkg::Options::=--force-confnew"
APT_TRUSTED_GPGD = "/etc/apt/trusted.gpg.d"
APT_CONFD = "/etc/apt/apt.conf.d"

#: Safe canonized paths
BASH_VARS = """\
MBD_SCRIPT="$(realpath "${0}")"  # Note: Older versions of realpath (<=wheezy) did not support options, just avoid them
MBD_BASE="${MBD_SCRIPT%/*/*}"\
"""


class Blocks(files.Dir):
    def __init__(self, type_):
        super().__init__()
        self.type = type_
        self.name = f"Sbuild-{self.type.capitalize()}-Blocks"
        self.default = ""

    def usage(self):
        default = self.default.strip()
        header = (
            f"Ordered sbuild {self.type} blocks (space-separated)\n"
            "\n"
            f"Default: {self.default}\n\n"
            "Available blocks:\n\n"
        )
        ml = max(len(name) for name in self.keys())  # "max length" formatting helper
        return header + "\n".join([f"{name:<{ml}}: [{'default' if name in default else 'extra'}] {block.description}" for name, block in self.items()])

    def validate(self, value):
        """Validate value from user space (string, space separated), return value as list"""
        eol = value.split()
        unknown = set(eol) - set(self.keys())
        if unknown:
            raise util.HTTPBadRequest(f"Unknown sbuild {self.type} block(s): {' '.join(unknown)}")
        return eol

    def line(self, value):
        self.validate(value)
        return f"{self.name}: {value}"


class ConfigBlocks(Blocks):
    def __init__(self):
        super().__init__("config")
        self.default = dist.SETUP["defaults"]["distribution"]["sbuild_config_blocks"]

    def configure(self, file_, blocks):
        """Add configured blocks to provided file"""
        for block in self.validate(blocks):
            file_.add(self[block].get(snippet=True))


class SetupBlocks(Blocks):
    def __init__(self):
        super().__init__("setup")
        self.default = dist.SETUP["defaults"]["distribution"]["sbuild_setup_blocks"]

    def configure(self, dir_, blocks):
        """Add configured blocks to provided dir"""
        for block in self.validate(blocks):
            dir_.add(self[block])


CONFIG_BLOCKS = ConfigBlocks()

CONFIG_BLOCKS.add(files.PerlModule(
    "set-default-path",
    description="Explicitly set $path (to sbuild's default value), so later config blocks may add to it",
    snippet="""\
$path = '/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games';
"""))

CONFIG_BLOCKS.add(files.PerlModule(
    "ccache",
    description="Update $path and $$build environment to support ccache",
    snippet="""\
$path = "/usr/lib/ccache:$path";
$$build_environment{'CCACHE_DIR'} = '%LIBDIR%/.ccache';
"""))

SETUP_BLOCKS = SetupBlocks()

SETUP_BLOCKS.add(files.BashScript(
    "apt-setup",
    description="Sets up APT for buildrequest (APT keys, optional https, sources and preferences)",
    snippet=f"""\
{BASH_VARS}

mkdir -p -v "{APT_TRUSTED_GPGD}"
cp -v "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_KEYS}" "{APT_TRUSTED_GPGD}/mini-buildd-buildrequest.asc"

if grep 'https://' "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_SOURCES_LIST}"; then
  {APT_GET} install ca-certificates apt-transport-https
  if [ -s "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_SSL_CERT}" ]; then
    cp -v "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_SSL_CERT}" /usr/local/share/ca-certificates/mini-buildd-repo.crt
    /usr/sbin/update-ca-certificates
  fi
fi

cp -v "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_SOURCES_LIST}" /etc/apt/sources.list
cat /etc/apt/sources.list

cp -v "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_PREFERENCES}" /etc/apt/preferences
cat /etc/apt/preferences
"""))

# * Using "timeout" to work around ``apt-parallel-bug```(see administrator.rst)
# * Using retry/sleep for occasional failures due to intermittent reprepro repository updates
SETUP_BLOCKS.add(files.ShScript(
    "apt-update",
    description="Update APT",
    snippet=f"""\
for N in 10 40 80 160 320; do
    if timeout ${{N}}s {APT_GET} update; then
       apt-cache policy
       exit 0
    fi
    printf "W: apt-update: Did not succeed after %s seconds (retrying)\n" "${{N}}" >&2
    sleep 10
done
exit 1
"""))

# All below need explicit opt-in
SETUP_BLOCKS.add(files.ShScript(
    "ccache",
    description="Installs ccache (together with the 'ccache' config block, this enables ccache for builds on the same instance)",
    snippet=f"""\
{APT_GET} install ccache
"""))

SETUP_BLOCKS.add(files.ShScript(
    "eatmydata",
    description="Install eatmydata and activate via LD_PRELOAD when possible (may improve speed)",
    snippet=f"""\
if {APT_GET} install eatmydata; then
    # eatmydata <= 26: Just one 'deb', and not multiarched
    EATMYDATA_SO=$(dpkg -L eatmydata | grep 'libeatmydata.so$') || true
    if [ -z "${{EATMYDATA_SO}}" ]; then
        # eatmydata >= 82: Has a library 'deb', and is multiarched
        EATMYDATA_SO=$(dpkg -L libeatmydata1 | grep 'libeatmydata.so$') || true
    fi
    if [ -n "${{EATMYDATA_SO}}" ]; then
        printf " ${{EATMYDATA_SO}}" >> /etc/ld.so.preload
    else
        printf "W: eatmydata: No *.so found (skipping)\\n" >&2
    fi
else
    printf "W: eatmydata: Not installable (skipping)\\n" >&2
fi
"""))


SETUP_BLOCKS.add(files.BashScript(
    "show-sbuildrc",
    description="Print generated '.sbuildrc' (informational only)",
    snippet=f"""\
{BASH_VARS}

cat "${{MBD_BASE}}/{CONFIG_DIR}/../.sbuildrc"
"""))


SETUP_BLOCKS.add(files.BashScript(
    "apt-key-binary",
    description="Convert APT keys to binary (needed for Debian <= jessie, Ubuntu <= yakketti) -- place between ``apt-setup`` and ``apt-update``",
    snippet=f"""\
{APT_GET} update
{APT_GET} install gnupg
gpg --dearmor "{APT_TRUSTED_GPGD}/mini-buildd-buildrequest.asc"
"""))

SETUP_BLOCKS.add(files.BashScript(
    "apt-key-add",
    description="Add APT keys via deprecated ``apt-key add`` (needed for Debian <= squeeze) -- place between ``apt-setup`` and ``apt-update``",
    snippet=f"""\
{BASH_VARS}

apt-key add "${{MBD_BASE}}/{CONFIG_DIR}/{CONFIG_APT_KEYS}"
"""))


def _setup_blocks_apt_option(name, option, desc):
    """Add a script that sets an APT option"""
    SETUP_BLOCKS.add(files.ShScript(
        name,
        description=f'Set APT option "{option}" ({desc}) -- place before ``apt-setup``',
        snippet=f'printf "%s\\n" "{option};" >"{APT_CONFD}/00mbd-{name}"\n'))


_setup_blocks_apt_option("apt-no-check-valid-until", "Acquire::Check-Valid-Until false", "needed when Release files are expired")
_setup_blocks_apt_option("apt-allow-unauthenticated", "APT::Get::AllowUnauthenticated true", "needed when APT keys are expired")
_setup_blocks_apt_option("apt-force-yes", "APT::Get::Force-Yes true", "compat(urold)")
_setup_blocks_apt_option("apt-no-cache", "Acquire::http::No-Cache true", "compat(urold)")
_setup_blocks_apt_option("apt-no-pipelining", "Acquire::http::Pipeline-Depth 0", "compat(urold)")


SETUP_BLOCKS.add(files.ShScript(
    "apt-clear",
    description="Try to clear any redundant APT data in ``/var/`` (experimental: can make APT work again if stuck)",
    snippet="""\
rm -v -r -f /var/lib/apt/lists/*
mkdir /var/lib/apt/lists/partial  # <= in lenny, apt would fail if missing
apt-get clean
"""))

SETUP_BLOCKS.add(files.ShScript(
    "bash-devices",
    description="Fixup /dev/fd|stdin|stdout|stderr. Needed for *building* bash only. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=327477 (compat(bash < 3.0-17 (2005))",
    snippet="""\
[ -e /dev/fd ] || ln -sv /proc/self/fd /dev/fd
[ -e /dev/stdin ] || ln -sv fd/0 /dev/stdin
[ -e /dev/stdout ] || ln -sv fd/1 /dev/stdout
[ -e /dev/stderr ] || ln -sv fd/2 /dev/stderr
"""))

SETUP_BLOCKS.add(files.BashScript(
    "schroot-shm",
    description="Fixup /dev/shm mount. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=728096 (compat(schroot < 1.6.10-3))",
    snippet="""\
# Pre 1.99.19, this used to live here: /etc/schroot/setup.d/15mini-buildd-workarounds

# '/dev/shm' might be a symlink to '/run/shm' in some
# chroots. Depending on the system mini-buildd/sbuild runs on
# and what chroot is being build for this may or may not lead to
# one or both of these problems:
#
# - shm not mounted properly in build chroot (leading to build errors when shm is used).
# - shm mount on *the host* gets overloaded (leading to an shm mount "leaking" on the host).
#
# This workaround
#
# - [in '/etc/schroot/mini-buildd/fstab-generic'] Removes shm form the generic sbuild mounts (avoids the mount leaking).
# - [here] Fixes /dev/shm to a directory in case it's not.
# - [here] Always mounts /dev/shm itself.
#
# Debian Bug: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=728096
#
# When this is fixed (and we have added a versioned depends on
# schroot accordingly), all of the above may be reverted again.
#
printf "=> Fixing up /dev/shm mount (Bug #728096):\\n"

if [ -L "/dev/shm" ]; then
  printf "Removing /dev/shm symlink...\\n"
  rm -v "/dev/shm"
fi
mkdir -v -p "/dev/shm"
mount -v -ttmpfs none "/dev/shm"
"""))

SETUP_BLOCKS.add(files.ShScript(
    "sun-java6-license",
    description="Accept sun-java6 licence (debconf) so we can build-depend on it (sun-java is no longer part of Debian since 2011)",
    snippet="""\
echo "sun-java6-bin shared/accepted-sun-dlj-v1-1 boolean true" | debconf-set-selections --verbose
echo "sun-java6-jdk shared/accepted-sun-dlj-v1-1 boolean true" | debconf-set-selections --verbose
echo "sun-java6-jre shared/accepted-sun-dlj-v1-1 boolean true" | debconf-set-selections --verbose
"""))


class SBuild():
    BUILDLOG_STATUS_REGEX = re.compile("^(Status|Lintian|Piuparts|Autopkgtest|Build-Time|Package-Time|Space|Build-Space): [^ ]+.*$")

    def __init__(self, breq, extra_args):
        self.breq = breq
        self.extra_args = extra_args
        self.bres = self.breq.gen_buildresult()
        self.call = None

    def run(self):
        """
        Run sbuild && update buildresult from build log

        .. note:: This will iterate all lines of the build log,
                  and parse out the selection of sbuild's summary status
                  we need.  In case the build log above does write the
                  same output like 'Status: xyz', sbuild's correct status
                  at the bottom will override this later.  Best thing,
                  though, would be if sbuild would eventually provide a
                  better way to get these values.
        """
        self.breq.untar(self.breq.dir_path())

        config_path = os.path.join(self.breq.dir_path(), CONFIG_DIR)
        sbuildrc_file_path = os.path.join(self.breq.dir_path(), ".sbuildrc")
        buildlog_file_path = os.path.join(self.breq.dir_path(), self.breq.dfn.buildlog(self.breq["architecture"]))

        #
        # Generate .sbuildrc
        #
        sbuildrc = files.PerlModule(".sbuildrc", placeholders={"LIBDIR": config.ROUTES["libdir"].path.new_sub([str(self.breq.dist.codename), self.breq["architecture"]], create=True).full})

        # From distribution config
        CONFIG_BLOCKS.configure(sbuildrc, self.breq.cget(CONFIG_BLOCKS.name, CONFIG_BLOCKS.default))

        # Custom script from distribution config
        sbuildrc.add_file(os.path.join(config_path, CONFIG_SBUILDRC_SNIPPET), description="Custom sbuild config (from Distribution)")

        sbuildrc.save_as(sbuildrc_file_path)

        #
        # Generate setup.d/
        #
        setup_d = files.Dir(os.path.join(self.breq.dir_path(), SETUP_D_DIR), enumerate_file_names=0)

        # From distribution config
        SETUP_BLOCKS.configure(setup_d, self.breq.cget(SETUP_BLOCKS.name, SETUP_BLOCKS.default))

        # Custom script from distribution config
        custom_buildrequest = os.path.join(config_path, CONFIG_CHROOT_SETUP_SCRIPT)
        if os.path.getsize(custom_buildrequest) > 0:
            setup_d.add(files.AutoScript(custom_buildrequest))

        setup_d.save()

        #
        # Generate command line
        #
        cmdline = [
            "sbuild",
            "--chroot-mode", "schroot",  # sbuild may eventually change default to "unshare"
            "--dist", self.breq["distribution"],
            "--arch", self.breq["architecture"],
            "--chroot", self.breq.schroot_name(),
            "--build-dep-resolver", self.breq.cget("Build-Dep-Resolver"),
            "--nolog",
            "--log-external-command-output",
            "--log-external-command-error",
            "--no-apt-update",
        ]

        for name in setup_d.keys():
            script = os.path.join(setup_d.path, name)
            cmdline += ["--chroot-setup-command", script]

        if self.breq.cget("Arch-All"):
            cmdline += ["--arch-all"]
        else:
            cmdline += ["--no-arch-all"]  # Be sure to set explicitly: sbuild >= 0.77 does not seem to use this as default any more?

        for dependency in self.breq.cget("Add-Depends", "").split(","):
            cmdline += ["--add-depends", dependency]

        lintian_check = self.breq.check_mode("lintian")
        if lintian_check.mode.value > 0:
            cmdline += ["--run-lintian"]
            cmdline += ["--lintian-opts", self.breq.check_extra_options("lintian")]
            if lintian_check.mode == lintian_check.Mode.WARNFAIL:
                cmdline += ["--lintian-opts", self.breq.cget("Lintian-Warnfail-Options")]
        else:
            cmdline += ["--no-run-lintian"]

        piuparts_check = self.breq.check_mode("piuparts")
        if piuparts_check.mode.value > 0:
            cmdline += ["--run-piuparts"]
            cmdline += ["--piuparts-opts", f"--schroot {self.breq.schroot_name()} --dpkg-force-confdef --warn-on-debsums-errors --warn-on-leftovers-after-purge --warn-on-others --keep-sources-list"]
            cmdline += ["--piuparts-opts", self.breq.check_extra_options("piuparts")]
        else:
            cmdline += ["--no-run-piuparts"]

        autopkgtest_check = self.breq.check_mode("autopkgtest")
        if autopkgtest_check.mode.value > 0:
            cmdline += ["--run-autopkgtest"]
            cmdline += ["--autopkgtest-opts", f"-- schroot {self.breq.schroot_name()}"]
            cmdline += ["--autopkgtest-root-args", ""]
        else:
            cmdline += ["--no-run-autopkgtest"]

        if "sbuild" in config.DEBUG:
            cmdline += ["--debug"]

        cmdline += self.extra_args

        cmdline += [self.breq.dfn.dsc()]

        # Generate Environment
        env = {
            "HOME": self.breq.dir_path(),
            "GNUPGHOME": config.ROUTES["home"].path.join(".gnupg"),
            "DEB_BUILD_OPTIONS": self.breq.cget("Deb-Build-Options", ""),
            "DEB_BUILD_PROFILES": self.breq.cget("Deb-Build-Profiles", ""),
        }

        # Run sbuild
        with util.fopen(buildlog_file_path, "w+") as buildlog, subprocess.Popen(
                cmdline,
                cwd=self.breq.dir_path(),
                env=call.taint_env(env),
                stdout=buildlog,
                stderr=subprocess.STDOUT) as self.call:
            LOG.debug("%s: Running sbuild from %s (pid=%s)", self.breq, self.breq.dir_path(), self.call.pid)
            retval = self.call.wait()

        # Update bres from buildlog
        self.bres.add_file(buildlog_file_path)
        with util.fopen(buildlog_file_path, errors="replace") as f:
            for line in f:
                if self.BUILDLOG_STATUS_REGEX.match(line):
                    # Example sbuild >= 0.91: "Lintian: successful (worst tag: warn)" (previously, there was no appendix)
                    LOG.debug("Build log line detected as build status: %s", line.strip())
                    key, _dummy, value = line.partition(":")
                    status = value.strip().partition(" ")[0]  # As lintian now (sbuild 0.91) has potential appendix, we need to use 1st word only as raw status
                    self.bres.cset("Sbuild-" + key, status)

        # Guarantee to set 'error' sbuild status if retval != 0 and 'Sbuild-Status' could not be parsed from buildlog
        if retval != 0 and not self.bres.cget("Sbuild-Status"):
            self.bres.cset("Sbuild-Status", "error")

        LOG.debug("%s: Sbuild finished: %s", self.breq, self.bres.cget('Sbuild-Status'))

        return self.bres

    def cancel(self, who):
        if self.call is not None:
            LOG.info("Terminating %s (sbuild process '%s')...", self.breq, self.call.pid)
            self.bres.cset("Canceled-By", who)
            self.call.terminate()

            # Wait some time (max 6 seconds) to see if terminate() succeeds
            for i in range(4):
                time.sleep(i)
                if self.call.poll() is not None:
                    return

            # I guess we should kill, then
            LOG.warning("Killing %s (sbuild process '%s')...", self.breq, self.call.pid)
            self.call.kill()



# Generated by Django 4.2.9 on 2024-01-07 20:49

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import mini_buildd.models
import mini_buildd.config
import mini_buildd.util


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('mini_buildd', '0021_migrate_suiteoption_extra_options'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='aptkey',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='architecture',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='architectureoption',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='archive',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='chroot',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='component',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='daemon',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='distribution',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='emailaddress',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='layout',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='prioritysource',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='remote',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='repository',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='source',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='subscription',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='suite',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='suiteoption',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='uploader',
            name='extra_options',
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_created',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_expires',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_id',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_long_id',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_name',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='architecture',
            name='name',
            field=models.CharField(max_length=50, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='archive',
            name='ping',
            field=models.FloatField(default=-1.0, editable=False),
        ),
        migrations.AlterField(
            model_name='archive',
            name='url',
            field=models.URLField(default='http://ftp.debian.org/debian/', max_length=512, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='btrfssnapshotchroot',
            name='chroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='architecture',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.architecture'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='debootstrap_command',
            field=models.CharField(default='/usr/sbin/debootstrap', max_length=255),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='debootstrap_extra_options',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='personality',
            field=models.CharField(blank=True, default='', editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='personality_override',
            field=models.CharField(blank=True, default='', max_length=50),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='component',
            name='name',
            field=models.CharField(max_length=50, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='allow_emails_to',
            field=models.CharField(default=mini_buildd.config.default_allow_emails_to, max_length=254),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='build_queue_size',
            field=models.IntegerField(default=mini_buildd.util.get_cpus),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='custom_archive_origin',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='email_address',
            field=mini_buildd.models.DaemonEmailField(default=mini_buildd.config.default_email_address, max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='ftpd_bind',
            field=models.CharField(default='tcp:port=8067', max_length=200),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='ftpd_options',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='gnupg_keyserver',
            field=models.CharField(default='keyserver.ubuntu.com', max_length=200),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='gnupg_template',
            field=mini_buildd.models.UnixTextField(default='Key-Type: RSA\nKey-Length: 4096\nExpire-Date: 0'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='identity',
            field=models.CharField(default=mini_buildd.config.default_identity, max_length=50),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='keep_build_results',
            field=models.IntegerField(default=5),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='show_last_builds',
            field=models.IntegerField(default=100),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='show_last_packages',
            field=models.IntegerField(default=500),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='wait_for_build_results',
            field=models.IntegerField(default=5),
        ),
        migrations.AlterField(
            model_name='dirchroot',
            name='chroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot'),
        ),
        migrations.AlterField(
            model_name='dirchroot',
            name='union_type',
            field=models.IntegerField(choices=[(0, 'aufs'), (1, 'overlayfs'), (2, 'unionfs'), (3, 'overlay')], default=3),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='add_depends',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='autopkgtest_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=0),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='base_source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='build_dep_resolver',
            field=models.IntegerField(choices=[(0, 'apt'), (1, 'aptitude'), (2, 'internal')], default=1),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='chroot_setup_script',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='deb_build_options',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='deb_build_profiles',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='internal_apt_priority',
            field=models.IntegerField(default=1),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_extra_options',
            field=models.CharField(default='--info', max_length=200),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=2),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='piuparts_extra_options',
            field=models.CharField(default='--info', max_length=200),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='piuparts_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=0),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='piuparts_root_arg',
            field=models.CharField(default='sudo', max_length=200),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuild_config_blocks',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuild_setup_blocks',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuildrc_snippet',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='emailaddress',
            name='address',
            field=models.EmailField(max_length=255, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='emailaddress',
            name='name',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='filechroot',
            name='chroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot'),
        ),
        migrations.AlterField(
            model_name='filechroot',
            name='compression',
            field=models.IntegerField(choices=[(0, 'no compression'), (1, 'gzip'), (2, 'bzip2'), (3, 'xz')], default=0),
        ),
        migrations.AlterField(
            model_name='layout',
            name='default_version',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%+1', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='experimental_default_version',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%+0', max_length=30),
        ),
        migrations.AlterField(
            model_name='layout',
            name='experimental_mandatory_version_regex',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%\\+0', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='mandatory_version_regex',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%\\+[1-9]', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='meta_distributions',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='layout',
            name='name',
            field=models.CharField(max_length=100, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='looplvmchroot',
            name='loop_size',
            field=models.IntegerField(default=100),
        ),
        migrations.AlterField(
            model_name='looplvmchroot',
            name='lvmchroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.lvmchroot'),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='chroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot'),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='filesystem',
            field=models.CharField(default='ext2', max_length=10),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='snapshot_size',
            field=models.IntegerField(default=4),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='volume_group',
            field=models.CharField(default='auto', max_length=80),
        ),
        migrations.AlterField(
            model_name='prioritysource',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='prioritysource',
            name='priority',
            field=models.IntegerField(default=1),
        ),
        migrations.AlterField(
            model_name='prioritysource',
            name='source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AlterField(
            model_name='remote',
            name='http',
            field=models.CharField(default='tcp:host=YOUR_REMOTE_HOST:port=8066', max_length=255, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_created',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_expires',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_id',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_long_id',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_name',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='remote',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='remote',
            name='wake_command',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='repository',
            name='allow_unauthenticated_uploads',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='external_home_url',
            field=models.URLField(blank=True),
        ),
        migrations.AlterField(
            model_name='repository',
            name='extra_uploader_keyrings',
            field=mini_buildd.models.UnixTextField(blank=True),
        ),
        migrations.AlterField(
            model_name='repository',
            name='identity',
            field=models.CharField(default='test', max_length=50, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='layout',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.layout'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='notify_changed_by',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='notify_maintainer',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='reprepro_morguedir',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='source',
            name='check_valid_until',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='source',
            name='codename',
            field=models.CharField(default='sid', max_length=60),
        ),
        migrations.AlterField(
            model_name='source',
            name='codeversion',
            field=models.CharField(blank=True, default='', max_length=50),
        ),
        migrations.AlterField(
            model_name='source',
            name='codeversion_override',
            field=models.CharField(blank=True, default='', editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name='source',
            name='description',
            field=models.CharField(blank=True, default='', editable=False, max_length=100),
        ),
        migrations.AlterField(
            model_name='source',
            name='extra_identifiers',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='source',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='source',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='source',
            name='origin',
            field=models.CharField(default='Debian', max_length=60),
        ),
        migrations.AlterField(
            model_name='source',
            name='remove_from_component',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='source',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='distribution',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='package',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='subscriber',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='suite',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='suite',
            name='name',
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_created',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_expires',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_id',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_long_id',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_name',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 4.2.9 on 2024-02-06 15:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0029_alter_source_options'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='suiteoption',
            name='auto_migrate_after',
        ),
    ]



# Generated by Django 4.2.11 on 2024-04-11 16:59

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mini_buildd", "0030_remove_suiteoption_auto_migrate_after"),
    ]

    operations = [
        migrations.AddField(
            model_name="daemon",
            name="expire_builds_after",
            field=models.DurationField(default=datetime.timedelta(days=5)),
        ),
        migrations.AddField(
            model_name="daemon",
            name="expire_events_after",
            field=models.DurationField(default=datetime.timedelta(days=336)),
        ),
    ]



"""mini-buildd's web application"""
import io
import logging
import os
import random

import django
import django.conf
import django.core.handlers.wsgi
import django.core.management

from mini_buildd import config, util

LOG = logging.getLogger(__name__)


def get(*args, **kwargs):
    return django.core.handlers.wsgi.WSGIHandler(*args, **kwargs)


class Call:
    """Run ``django-admin`` commands with error handling and smart logging"""

    def _log(self, log, msg):
        log("[django-admin %s] %s", self.name, msg)

    def _logf(self, log, f):
        f.seek(0)
        for line in f:
            self._log(log, line.strip())

    def __init__(self, name, *args, **kwargs):
        self.name = name
        self.error = None
        self.stdout, self.stderr = io.StringIO(), io.StringIO()

        try:
            django.core.management.call_command(self.name, *args, **kwargs, stdout=self.stdout, stderr=self.stderr)
        except BaseException as e:
            self.error = e
            raise
        finally:
            self._logf(LOG.info, self.stdout)
            self._logf(LOG.warning, self.stderr)
            if self.error is not None:
                self._log(LOG.error, str(self.error))


def migrate():
    try:
        Call("migrate", interactive=False, run_syncdb=True)
    except django.db.OperationalError:
        LOG.info("Retrying migration with '--fake-initial'...")
        Call("migrate", interactive=False, run_syncdb=True, fake_initial=True)


def init():
    migrate()
    Call("check")
    config.SqlitePath().fix_perms()

    # For temporary debugging/check settings only
    # Call("diffsettings", output="unified")


def set_admin_password(password):
    """
    Set the password for the administrator

    :param password: The password to use.
    :type password: string
    """
    # This import needs the django app to be already configured (since django 1.5.2)
    from django.contrib.auth import models  # pylint: disable=import-outside-toplevel

    try:
        user = models.User.objects.get(username="admin")
        LOG.info("Updating 'admin' user's password...")
        user.set_password(password)
        user.save()
    except models.User.DoesNotExist:
        LOG.info("Creating 'admin' user with password...")
        models.User.objects.create_superuser("admin", "root@localhost", password)


def remove_system_artifacts():
    """Bulk-remove all model instances that might have produced cruft on the system (outside mini-buildd's home)"""
    for chroot in util.models().Chroot.mbd_get_prepared():
        try:
            chroot.Admin.mbd_remove(chroot)
        except BaseException as e:
            LOG.warning("Failed to remove %s: %s", chroot, e)


def loaddata(file_name):
    Call("loaddata", file_name)


def dumpdata(app_path):
    LOG.debug("Dumping data for: %s", app_path)
    Call("dumpdata", app_path, indent=2, format="json")


# Formerly in django_settings.py
MBD_INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.admin",
    "django.contrib.sessions",
    "django.contrib.admindocs",
    "django.contrib.staticfiles",
    "mini_buildd",
)


class SMTPCreds():
    """
    SMTP creds string parser -- format "USER:PASSWORD@smtp|ssmtp://HOST:PORT"

    >>> d = SMTPCreds(":@smtp://localhost:25")
    >>> (d.user, d.password, d.protocol, d.host, d.port)
    ('', '', 'smtp', 'localhost', 25)
    >>> d = SMTPCreds("kuh:sa:ck@smtp://colahost:44")
    >>> (d.user, d.password, d.protocol, d.host, d.port)
    ('kuh', 'sa:ck', 'smtp', 'colahost', 44)
    """

    def __init__(self, creds):
        self.creds = creds
        at = creds.partition("@")

        usrpass = at[0].partition(":")
        self.user = usrpass[0]
        self.password = usrpass[2]

        smtp = at[2].partition(":")
        self.protocol = smtp[0]

        hopo = smtp[2].partition(":")
        self.host = hopo[0][2:]
        self.port = int(hopo[2])


def gen_django_secret_key():
    # use same randomize-algorithm as in "django/core/management/commands/startproject.py"
    return "".join([random.choice("abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)") for _i in range(50)])


def get_django_secret_key(home):
    """
    Create django's SECRET_KEY *once* and/or returns it

    :param home: mini-buildd's home directory.
    :type home: string
    :returns: string -- the (created) key.
    """
    secret_key_filename = os.path.join(home, ".django_secret_key")

    # the key to create or read from file
    secret_key = ""

    if not os.path.exists(secret_key_filename):
        # use same randomize-algorithm as in "django/core/management/commands/startproject.py"
        secret_key = gen_django_secret_key()
        with os.fdopen(os.open(secret_key_filename, os.O_CREAT | os.O_WRONLY, 0o600), "w") as secret_key_fd:
            secret_key_fd.write(secret_key)
    else:
        with util.fopen(secret_key_filename, "r") as existing_file:
            secret_key = existing_file.read()

    return secret_key


def configure(smtp_string):
    """Configure django"""
    LOG.debug("Setting up django...")

    smtp = SMTPCreds(smtp_string)
    debug = "webapp" in config.DEBUG

    django.conf.settings.configure(
        DEBUG=debug,

        # django 4.0: https://docs.djangoproject.com/en/5.2/releases/4.0/#csrf-trusted-origins-changes-4-0
        # Trust --hostname value for http and https: Fixes django logins on a (80/443) proxy setup
        CSRF_TRUSTED_ORIGINS=[f"https://{config.HOSTNAME_FQDN}", f"http://{config.HOSTNAME_FQDN}"],

        DEFAULT_AUTO_FIELD="django.db.models.AutoField",  # django 3.2: https://docs.djangoproject.com/en/3.2/releases/3.2/#customizing-type-of-auto-created-primary-keys

        ALLOWED_HOSTS=["*"],

        EMAIL_HOST=smtp.host,
        EMAIL_PORT=smtp.port,
        EMAIL_USE_TLS=smtp.protocol == "ssmtp",
        EMAIL_HOST_USER=smtp.user,
        EMAIL_HOST_PASSWORD=smtp.password,

        AUTH_PASSWORD_VALIDATORS=[
            {
                "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
            },
            {
                "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
                "OPTIONS": {
                    "min_length": 8,
                }
            },
            {
                "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
            },
            {
                "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
            },
        ],

        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
                "NAME": config.SqlitePath().path,
            }},

        TIME_ZONE="Etc/UTC",  # Default time zone: Canonical UTC. Better than django's default 'America/Chicago'.
        USE_TZ=True,          # Django: "stores datetime information in UTC in the database": that's definitely what we want.
        USE_L10N=True,

        SECRET_KEY=get_django_secret_key(config.ROUTES["home"].path.join()),
        ROOT_URLCONF="mini_buildd.urls",
        STATIC_URL=config.URIS["static"]["static"].uri,
        ACCOUNT_ACTIVATION_DAYS=3,
        LOGIN_URL=config.URIS["accounts"]["login"].uri,
        LOGOUT_REDIRECT_URL="/",
        LOGIN_REDIRECT_URL="/",

        MIDDLEWARE=[
            "django.middleware.common.CommonMiddleware",
            "django.contrib.sessions.middleware.SessionMiddleware",
            "django.middleware.csrf.CsrfViewMiddleware",
            "django.contrib.auth.middleware.AuthenticationMiddleware",
            "django.contrib.messages.middleware.MessageMiddleware",
            "mini_buildd.views.ExceptionMiddleware"
        ],

        INSTALLED_APPS=MBD_INSTALLED_APPS,
        TEMPLATES=[
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "DIRS": [
                    f"{config.PY_PACKAGE_PATH}/mini_buildd/templates"
                ],
                "APP_DIRS": True,
                "OPTIONS": {
                    "debug": debug,
                    "context_processors": [
                        #: Django defaults as per ``django-admin startproject dummy`` (django 2.2.20)
                        "django.template.context_processors.debug",
                        "django.template.context_processors.request",
                        "django.contrib.auth.context_processors.auth",
                        "django.contrib.messages.context_processors.messages",
                        #: Custom
                        "mini_buildd.views.context",
                    ],
                    "builtins": [
                        "mini_buildd.builtins",
                        "django.templatetags.static",
                    ],
                },
            },
            # {
            #     "BACKEND": "django.template.backends.jinja2.Jinja2",
            #     "DIRS": [
            #         f"{config.PY_PACKAGE_PATH}/mini_buildd/jinja2",
            #     ],
            # },
        ])

    django.setup()


def pseudo_configure(sqlite_path=":memory:"):
    """
    Pseudo-configure django (minimal setup to access models and run admin)

    Used internally only (where you need mini-buildd's model classes, but no actual instances): sphinx doc, run doctest, django SQL migrations, ...
    """
    django.conf.settings.configure(
        SECRET_KEY=gen_django_secret_key(),
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
                "NAME": sqlite_path,
            }
        },
        MIDDLEWARE=[
            "django.contrib.sessions.middleware.SessionMiddleware",
            "django.contrib.auth.middleware.AuthenticationMiddleware",
            "django.contrib.messages.middleware.MessageMiddleware",
        ],
        TEMPLATES=[
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "OPTIONS": {
                    "context_processors": [
                        "django.contrib.auth.context_processors.auth",
                        "django.contrib.messages.context_processors.messages",
                    ],
                },
            },
        ],
        INSTALLED_APPS=MBD_INSTALLED_APPS)

    django.setup()



import abc
import argparse
import configparser
import inspect
import logging
import logging.handlers
import os
import pwd
import sys
import warnings

import argcomplete

from mini_buildd import net, util

LOG = logging.getLogger(__name__)


warnings.formatwarning = lambda message, category, filename, lineno, line=None: f"{filename}:{lineno}: {category.__name__}: {message}"


def log_test():
    """For debugging only"""
    LOG.debug("debug")
    LOG.info("info")
    LOG.warning("warning")
    LOG.error("error")
    LOG.critical("critical")


def log_info():
    """For debugging only"""
    for logger in [logging.getLogger()] + [logging.getLogger(name) for name in logging.root.manager.loggerDict]:  # pylint:disable=no-member
        print(f"{logger}: {logger.handlers}")


class StdErrHandler(logging.StreamHandler):
    #: Log levels that we use throughout the code (python defaults) mapped to numerical counterparts used in syslog/systemd
    SYSTEMD_PREFIX = {} if sys.stderr.isatty() else {
        logging.getLevelNamesMapping()["CRITICAL"]: "<2>",
        logging.getLevelNamesMapping()["ERROR"]: "<3>",
        logging.getLevelNamesMapping()["WARNING"]: "<4>",
        logging.getLevelNamesMapping()["INFO"]: "<6>",
        logging.getLevelNamesMapping()["DEBUG"]: "<7>",
    }

    #: Generic log format
    LOG_FORMAT = "{systemd_prefix}{levelname[0]}: {message} [{threadName}({thread}), {name}({filename}:{lineno})]"

    def __init__(self):
        super().__init__(stream=sys.stderr)
        self.setFormatter(logging.Formatter(self.LOG_FORMAT, style="{"))

    def emit(self, record):
        record.systemd_prefix = self.SYSTEMD_PREFIX.get(record.levelno, "")
        return super().emit(record)


class DputCf():
    """Guess possible mini-buildd targets and their URL endpoints"""

    DEFAULT_PATH = "~/.dput.cf"

    def __init__(self, config_path=DEFAULT_PATH):
        self.config = os.path.expanduser(config_path)
        dput_cf = configparser.ConfigParser(interpolation=None)
        dput_cf.read(self.config)

        self.parsed = {}
        for section in dput_cf.sections():
            cfg = dput_cf[section]
            method = cfg.get("method")
            host, dummy, ftp_port = cfg.get("fqdn").partition(":")
            if section.startswith("mini-buildd") and method in ["ftp", "ftps"]:
                http_port = int(ftp_port) - 1
                http_method = "https" if method == "ftps" else "http"
                self.parsed[section] = {}
                self.parsed[section]["http_url"] = f"{http_method}://{host}:{http_port}"
                self.parsed[section]["ftp_url"] = f"{method}://{host}:{ftp_port}"

    def first_target(self):
        return next(iter(self.parsed))

    def target_completer(self, **_kwargs):
        return self.parsed.keys()

    def first_http_url(self):
        return self.parsed[self.first_target()]["http_url"]

    def http_url_completer(self, **_kwargs):
        return [target["http_url"] for target in self.parsed.values()]

    def get_target_ftp_url(self, target):
        return self.parsed[target]["ftp_url"]

    def get_target_http_url(self, target):
        return self.parsed[target]["http_url"]


class ArgumentDefaultsRawTextHelpFormatter(argparse.RawTextHelpFormatter, argparse.ArgumentDefaultsHelpFormatter):
    """Custom argparse (for mini-buildd[-tool]) help formatter (mixin): We like to use raw text, but also have default values shown"""


class CLI():
    LOG_LEVEL_DEFAULT = ["WARNING", "mini_buildd.INFO"]

    @classmethod
    def log_level_parse(cls, value, set_level=False):
        logger, dummy, level = value.rpartition(".")
        if level not in logging.getLevelNamesMapping():
            raise argparse.ArgumentTypeError(f"{level}: Invalid log level")
        # print(f"PSD: {logger or 'root'}={level}")
        if set_level:
            logging.getLogger(logger).setLevel(level)
            # print(f"SET: {logger or 'root'}={level}")
        return value

    @classmethod
    def log_level_completer(cls, **_kwargs):
        return logging.getLevelNamesMapping().keys()

    def __init__(self, prog, description, epilog=None, run_as_mini_buildd=False):
        self.prog = prog
        self.args = None
        self._user = pwd.getpwuid(os.getuid())[0]
        self.run_as_mini_buildd = run_as_mini_buildd

        self.parser = argparse.ArgumentParser(prog=prog, description=description, epilog=epilog,
                                              formatter_class=ArgumentDefaultsRawTextHelpFormatter)
        self.parser.add_argument("--version", action="version", version=util.__version__)
        self.parser.add_argument("-l", "--log-level", action="append", type=self.log_level_parse,
                                 help=(
                                     f"set log level (if none given, {self.LOG_LEVEL_DEFAULT} is applied by default).\n"
                                     "May also be given as ``<logger>.<level>`` to effect individual loggers only,\n"
                                     "and multiple times (check existing logs for actual logger names)."
                                 )).completer = CLI.log_level_completer
        if run_as_mini_buildd:
            self.parser.add_argument("-U", "--dedicated-user", action="store", default="mini-buildd",
                                     help="Force a custom dedicated user name (to run as a different user than 'mini-buildd').")

    @classmethod
    def _add_endpoint(cls, parser):
        parser.add_argument("endpoint", action="store",
                            metavar="ENDPOINT",
                            help=f"HTTP target endpoint: {inspect.getdoc(net.ClientEndpoint)}").completer = DputCf().http_url_completer

    @classmethod
    def _add_subparser(cls, subparser, cmd, doc):
        return subparser.add_parser(cmd, help=doc, formatter_class=ArgumentDefaultsRawTextHelpFormatter)

    @abc.abstractmethod
    def runcli(self):
        pass

    def run(self):
        argcomplete.autocomplete(self.parser)
        self.args = self.parser.parse_args()

        # As 'append' would append to default, we handle default values manually (https://bugs.python.org/issue16399)
        if self.args.log_level is None:
            self.args.log_level = self.LOG_LEVEL_DEFAULT

        logging.root.handlers = [StdErrHandler()]
        for log_level in self.args.log_level:
            self.log_level_parse(log_level, set_level=True)

        # Raise exceptions in a debug setup only (see https://docs.python.org/3.5/howto/logging.html#exceptions-raised-during-logging)
        logging.raiseExceptions = logging.root.isEnabledFor(logging.DEBUG)

        # Always capture possible python warnings to py.warning logger
        logging.captureWarnings(True)

        # If not set explicitly otherwise (PYTHONWARNINGS, -W): Default to ignore python warnings
        if not sys.warnoptions:
            warnings.simplefilter("ignore")

        try:
            if self.run_as_mini_buildd and self.args.dedicated_user != self._user:
                raise util.HTTPUnauthorized(f"Run as user 'mini-buildd' only (use '--dedicated-user={self._user}' if you really want this, will write to that user's $HOME!)")

            self.runcli()
        except BaseException as e:
            util.log_exception(LOG, f"{self.prog} failed (try '--log-level DEBUG')", e, level=logging.ERROR)
            sys.exit(1)



import logging

import django.conf.urls
import django.views.generic.base
from django.urls import include, path, re_path

from mini_buildd import admin, api, config, views

LOG = logging.getLogger(__name__)

home_view = django.views.generic.base.RedirectView.as_view(url=config.URIS["events"]["view"].uri)


urlpatterns = [
    # Redirect
    path("", home_view),
    path("mini_buildd/", home_view),

    # /admin
    path(config.URIS["admin"]["view"].django(), admin.site.urls),

    # /accounts/
    path(config.URIS["accounts"]["register"].django(), views.AccountRegisterView.as_view(), name="register"),
    path(config.URIS["accounts"]["activate"].django() + "<uidb64>/<token>/", views.AccountActivateView.as_view(), name="activate"),
    path(config.URIS["accounts"]["profile"].django(), views.AccountProfileView.as_view(), name="profile"),
    path(config.URIS["accounts"]["null"].django(), views.AccountNullView.as_view(), name="null"),
    path(config.URIS["accounts"]["base"].django(), include("django.contrib.auth.urls")),

    # /mini_buildd/
    re_path(config.URIS["events"]["view"].django(), views.EventsView.as_view(), kwargs={"route": "events", "name": "events"}, name="events"),
    re_path(config.URIS["events"]["dir"].django(), views.DirView.as_view(), kwargs={"route": "events", "name": "events-dir"}, name="events-dir"),
    re_path(config.URIS["events"]["static"].django(), views.DefaultView.as_view(), name="events-static"),
    re_path(config.URIS["builds"]["dir"].django(), views.DirView.as_view(), kwargs={"route": "builds", "name": "builds-dir"}, name="builds-dir"),
    re_path(config.URIS["builds"]["static"].django(), views.DefaultView.as_view(), name="builds-static"),
    re_path(config.URIS["repositories"]["view"].django(), views.RepositoriesView.as_view(), kwargs={"route": "repositories", "name": "repositories"}, name="repositories"),
    re_path(config.URIS["repositories"]["dir"].django(), views.DirView.as_view(), kwargs={"route": "repositories", "name": "repositories-dir"}, name="repositories-dir"),
    re_path(config.URIS["repositories"]["static"].django(), views.DefaultView.as_view(), name="repositories-static"),
    path(config.URIS["builders"]["view"].django(), views.BuildersView.as_view(), kwargs={"route": "builders", "name": "builders"}, name="builders"),
    path(config.URIS["crontab"]["view"].django(), views.DefaultView.as_view(), kwargs={"route": "crontab", "name": "crontab"}, name="crontab"),
    path(config.URIS["api"]["view"].django(), views.DefaultView.as_view(), kwargs={"route": "api", "name": "api"}, name="api"),
    path(config.URIS["sitemap"]["view"].django(), views.DefaultView.as_view(), kwargs={"route": "sitemap", "name": "sitemap"}, name="sitemap"),
    path(config.URIS["manual"]["view"].django(), views.ManualView.as_view(), kwargs={"route": "manual", "name": "manual"}, name="manual"),
    path(config.URIS["manual"]["static"].django(), views.ManualView.as_view(), kwargs={"route": "manual", "name": "manual-static"}, name="manual-static"),
    path(config.URIS["setup"]["view"].django(), views.SetupView.as_view(), kwargs={"route": "setup", "name": "setup"}, name="setup"),
]

# /mini_buildd/api/<call>/
for _NAME, _CALL in api.CALLS.items():
    _uri = config.Uri(_CALL.uri())
    LOG.debug("Adding API route: %s: %s", _NAME, _uri)
    urlpatterns.append(path(_uri.django(), views.APIView.as_view(), kwargs={"call": _CALL}, name=f"apicall_{_NAME}"))

django.conf.urls.handler400 = views.ExceptionMiddleware.bad_request
django.conf.urls.handler403 = views.ExceptionMiddleware.permission_denied
django.conf.urls.handler404 = views.ExceptionMiddleware.page_not_found
django.conf.urls.handler500 = views.ExceptionMiddleware.server_error



import enum
import logging
import os
import socket
import stat
import sys

from mini_buildd import files, util

LOG = logging.getLogger(__name__)

#: This should never ever be changed
CHAR_ENCODING = "UTF-8"

#: Compute python-version dependent install path
PY_PACKAGE_PATH = f"/usr/lib/python{sys.version_info[0]}/dist-packages"

#: Global constant object to use when signalling shutdown via message queues
SHUTDOWN = "SHUTDOWN"

#: Config for HTTPD (twisted) and event queue clients
MIN_HTTPD_THREADS = 10
MAX_HTTPD_THREADS = 100
MAX_EVENTS_QUEUE_CLIENTS = 70

#: HTTPD log file names
ACCESS_LOG_FILE = "access.log"

#: (Debian package) path for the internal package templates
PACKAGE_TEMPLATES = "/usr/share/mini-buildd/package-templates"

#: Sample message used for instance handshakes
HANDSHAKE_MESSAGE = "Hand Shake"


class AuthType(enum.Enum):
    NONE = enum.auto()
    LOGIN = enum.auto()
    STAFF = enum.auto()
    ADMIN = enum.auto()


class Auth():
    def __init__(self, auth):
        self.auth = auth

    def __str__(self):
        return self.auth.name

    def is_authorized(self, user):
        """Check if django user is authorized"""
        def active_login():
            return user.is_authenticated and user.is_active

        if self.auth == AuthType.NONE:
            return True
        if user is None:
            return False
        if self.auth == AuthType.LOGIN and active_login():
            return True
        if self.auth == AuthType.STAFF and active_login() and user.is_staff:
            return True
        if self.auth == AuthType.ADMIN and active_login() and user.is_superuser:
            return True
        return False


#: Auth shortcuts
AUTH_NONE = Auth(AuthType.NONE)
AUTH_LOGIN = Auth(AuthType.LOGIN)
AUTH_STAFF = Auth(AuthType.STAFF)
AUTH_ADMIN = Auth(AuthType.ADMIN)


class Uri:
    """URI string with some convenience functionality"""

    DJANGO_PATH_REGEX = r"(?:(?P<path>.*))?"

    def __init__(self, uri, auth=AUTH_NONE, django_with_path=False):
        self.uri = uri

        #: Auth
        self.auth = auth

        #: Non-static: Add 'path' to django URI
        self.django_with_path = django_with_path

    def __str__(self):
        return self.uri

    def django(self):
        return self.uri.lstrip("/") + (self.DJANGO_PATH_REGEX if self.django_with_path else "")

    def twisted(self):
        return self.uri.strip("/")

    def join(self, *args, prefix=""):
        return os.path.join(prefix, self.uri, *args)

    def url_join(self, *args, endpoint=None):
        ep = util.http_endpoint() if endpoint is None else endpoint
        return ep.geturl(path=self.join(*args))

    @classmethod
    def uri2view(cls, uri):
        return uri.replace("static", "mini_buildd", 1)

    def to_view(self):
        return Uri(self.uri2view(self.uri))


#: Static URI dict (by route name)
#: 'view': The default (django).
#: 'dir': Directory Listing (django).
#: 'static': Static delivery (twisted).
URIS = {
    "static": {
        "static": Uri("/static/static/"),
    },
    "setup": {
        "view": Uri("/mini_buildd/setup/"),
    },
    "events": {
        "view": Uri("/mini_buildd/events/", django_with_path=True),
        "dir": Uri("/mini_buildd/events-dir/", django_with_path=True),
        "static": Uri("/static/events/", django_with_path=True),
        "attach": Uri("/static/events.attach"),
    },
    "builds": {
        "dir": Uri("/mini_buildd/builds-dir/", django_with_path=True),
        "static": Uri("/static/builds/", django_with_path=True),
    },
    "repositories": {
        "view": Uri("/mini_buildd/repositories/", django_with_path=True),
        "dir": Uri("/mini_buildd/repositories-dir/", django_with_path=True),
        "static": Uri("/repositories/", django_with_path=True),
    },
    "builders": {
        "view": Uri("/mini_buildd/builders/"),
    },
    "crontab": {
        "view": Uri("/mini_buildd/crontab/"),
    },
    "api": {
        "view": Uri("/mini_buildd/api/"),
    },
    "admin": {
        "view": Uri("/admin/"),
    },
    "manual": {
        "static": Uri("/static/manual/"),
        "view": Uri("/mini_buildd/manual/"),
    },
    "sitemap": {
        "view": Uri("/mini_buildd/sitemap/"),
    },
    "accounts": {
        "base": Uri("/accounts/"),               # used to include django standard auth urls under, see urls.py
        "login": Uri("/accounts/login/"),        # is standard django, but we need this URI when we login internally, see net.py
        "register": Uri("/accounts/register/"),
        "activate": Uri("/accounts/activate/"),
        "profile": Uri("/accounts/profile/"),
        "null": Uri("/accounts/null/"),
    },
    "homepage": {
        "view": Uri("http://mini-buildd.installiert.net/"),
    },
}


class Route:
    """Link a path (may be None, may be run-time) to uris (may be empty)"""

    def __init__(self, path=None, uris=None):
        self.path = path
        self.uris = {} if uris is None else uris

    def get_uri(self, typ, path=""):
        """Get URI for path with given type (like static, dir, view)"""
        uri = self.uris.get(typ)
        return uri.join(path) if uri else None


class Routes(dict):
    def _add(self, name, path):
        self[name] = Route(path, uris=URIS.get(name))

    def __init__(self, home):
        super().__init__()
        self.home = home

        def homepth(path):
            return os.path.join(self.home, path)

        self._add("home", files.Path(self.home))
        self._add("etc", files.Path(homepth("etc"), create=True))
        self._add("setup", None)
        self._add("static", files.Path(f"{PY_PACKAGE_PATH}/mini_buildd/static"))

        # 'var/shared': Shared with chroots
        self._add("builds", files.Path(homepth("var/shared/builds"), create=True))   # Build directory
        self._add("libdir", files.Path(homepth("var/shared/libdir"), create=True))   # Shared libdir: Can optionally be used for persistent shared data (like ccache)
        self._add("debug", files.Path(homepth("var/shared/debug"), create=True))     # Shared directory for optional manual (sbuild) debugging

        self._add("events", files.Path(homepth("var/events"), create=True))
        self._add("event_hooks", files.Path(homepth("etc/event-hooks"), create=True))

        self._add("repositories", files.Path(homepth("repositories"), create=True))
        self._add("chroots", files.Path(homepth("var/chroots"), create=True))
        self._add("builders", None)
        self._add("crontab", files.Path(homepth("var/crontab"), create=True))
        self._add("sitemap", None)
        self._add("api", None)

        self._add("log", files.Path(homepth("var/log"), create=True))

        self._add("tmp", files.Path(homepth("var/tmp"), create=True))
        self._add("incoming", files.Path(homepth("incoming"), create=True))

        self._add("manual", files.Path(os.path.realpath("/usr/share/doc/mini-buildd/html")))


#: Variable items (see mini-buildd main script)
DEBUG = []
HOSTNAME = socket.gethostname()
HOSTNAME_FQDN = socket.getfqdn()
#: HTTP endpoints strings
HTTP_ENDPOINTS = ["tcp6:port=8066"]
ROUTES = {}


def default_ftp_endpoint():
    """Compute default ftp endpoint string from main http endpoint"""
    port = util.http_endpoint().options.get("port")
    return util.http_endpoint().description.replace(f"port={port}", f"port={int(port) + 1}")


def default_identity():
    return HOSTNAME


def default_email_address():
    return f"mini-buildd@{HOSTNAME_FQDN}"


def default_allow_emails_to():
    return f".*@{HOSTNAME_FQDN}"


class SqlitePath():
    def __init__(self):
        self.path = ROUTES["home"].path.join("config.sqlite")

    def fix_perms(self):
        LOG.debug("Fixing permissions on %s...", self.path)
        os.chmod(self.path, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP)



# Generated by Django 4.2.9 on 2024-01-07 17:32

from django.db import migrations

from . import mbd_get_extra_option


def copy_extra_options(apps, _schema_editor):
    SuiteOption = apps.get_model("mini_buildd", "SuiteOption")
    for suite_option in SuiteOption.objects.all():
        suite_option.rollback = int(mbd_get_extra_option(suite_option, "Rollback", "0"))
        suite_option.save()


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0020_add_suiteoption_extra_options'),
    ]

    operations = [
        migrations.RunPython(copy_extra_options),
    ]
